# Copyright (c) 2017 Huawei Technologies Co., Ltd.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

from oslo_log import versionutils
from oslo_policy import policy


# This check string is reserved for actions that require the highest level of
# authorization across projects to operate the deployment. They're allowed to
# create, read, update, or delete any system-wide resource such as
# share types, share group types, storage pools, etc. They can also operate on
# project-specific resources where applicable (e.g., cleaning up shares or
# snapshots).
ADMIN = 'rule:context_is_admin'

# This check string is the primary use case for typical end-users, who are
# working with resources that belong within a project (e.g., managing shares or
# share replicas). These users don't require all the authorization that
# administrators typically have.
PROJECT_MEMBER = 'rule:project-member'

# This check string should only be used to protect read-only project-specific
# resources. It should not be used to protect APIs that make writable changes
# (e.g., updating a share or snapshot). This persona is useful for someone who
# needs access for auditing or even support.
PROJECT_READER = 'rule:project-reader'

ADMIN_OR_PROJECT_MEMBER = f'({ADMIN}) or ({PROJECT_MEMBER})'
ADMIN_OR_PROJECT_READER = f'({ADMIN}) or ({PROJECT_READER})'

# Old, "unscoped", deprecated check strings to be removed. Do not use these
# in default RBAC any longer. These can be removed after "enforce_scope"
# defaults to True in oslo.policy
RULE_ADMIN_OR_OWNER = 'rule:admin_or_owner'
RULE_ADMIN_API = 'rule:admin_api'
RULE_DEFAULT = 'rule:default'

deprecation_msg = ("The `context_is_admin` check is superseded by more "
                   "specific check strings that consume project "
                   "scope attributes from keystone tokens.")
DEPRECATED_CONTEXT_IS_ADMIN = policy.DeprecatedRule(
    name='context_is_admin',
    check_str='role:admin',
    deprecated_reason=deprecation_msg,
    deprecated_since=versionutils.deprecated.WALLABY
)

rules = [
    # ***Default OpenStack scoped personas*** #
    policy.RuleDefault(
        name='project-admin',
        check_str='role:admin and '
                  'project_id:%(project_id)s',
        description='Project scoped Administrator',
        scope_types=['project']),
    policy.RuleDefault(
        name='project-member',
        check_str='role:member and '
                  'project_id:%(project_id)s',
        description='Project scoped Member',
        scope_types=['project']),
    policy.RuleDefault(
        name='project-reader',
        check_str='role:reader and '
                  'project_id:%(project_id)s',
        description='Project scoped Reader',
        scope_types=['project']),

    # ***Special personas for Manila*** #
    policy.RuleDefault(
        name='context_is_admin',
        check_str='role:admin',
        description='Privileged users checked via "context.is_admin"',
        deprecated_rule=DEPRECATED_CONTEXT_IS_ADMIN,
        scope_types=['project']),

    # ***Legacy/deprecated unscoped rules*** #
    # can be removed after "enforce_scope" defaults to True in oslo.policy
    policy.RuleDefault(
        name='admin_or_owner',
        check_str='is_admin:True or project_id:%(project_id)s',
        description='Administrator or Member of the project'),
    policy.RuleDefault(
        name='default',
        check_str=RULE_ADMIN_OR_OWNER,
        description='Default rule for most non-Admin APIs'),
    policy.RuleDefault(
        name='admin_api',
        check_str='is_admin:True',
        description='Default rule for most Admin APIs.'),
]


def list_rules():
    return rules



# Copyright (c) 2022 MacroSAN Technologies Co., Ltd.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re

from oslo_config import cfg
from oslo_log import log
from oslo_utils import units

from manila import exception
from manila.i18n import _
from manila.share.drivers.macrosan import macrosan_constants as constants
from manila.share.drivers.macrosan import rest_helper
from manila.share import utils as share_utils

CONF = cfg.CONF

LOG = log.getLogger(__name__)


class MacrosanHelper(object):
    def __init__(self, configuration):
        self.configuration = configuration
        self.rest = rest_helper.RestHelper(self.configuration)
        self.snapshot_support = False
        self.replication_support = False
        self.pools = self.configuration.macrosan_share_pools

    def check_share_service(self):
        nfs_service = self.rest._get_nfs_service_status()
        if nfs_service['serviceStatus'] not in [constants.NFS_NON_CONFIG,
                                                constants.NFS_ENABLED,
                                                constants.NFS_DISABLED]:
            raise exception.MacrosanBackendExeption(
                reason=_("nfs service exception. Please check backend"))
        elif nfs_service['serviceStatus'] == constants.NFS_NON_CONFIG:
            self.rest._config_nfs_service()
            self.rest._start_nfs_service()
        elif nfs_service['serviceStatus'] == constants.NFS_DISABLED:
            if (nfs_service['nfs3Status'] == constants.NFS_NON_SUPPORTED and
                    nfs_service['nfs4Status'] == constants.NFS_NON_SUPPORTED):
                self.rest._config_nfs_service()
            self.rest._start_nfs_service()
        else:
            if (nfs_service['nfs3Status'] == constants.NFS_NON_SUPPORTED and
                    nfs_service['nfs4Status'] == constants.NFS_NON_SUPPORTED):
                self.rest._config_nfs_service()

        cifs_status = self.rest._get_cifs_service_status()
        if cifs_status == constants.CIFS_EXCEPTION:
            raise exception.MacrosanBackendExeption(
                reason=_("cifs service exception.Please check backend"))
        elif cifs_status == constants.CIFS_NON_CONFIG:
            """need config first, then start service"""
            self.rest._config_cifs_service()
            self.rest._start_cifs_service()
        elif cifs_status == constants.CIFS_DISABLED:
            self.rest._start_cifs_service()
            status = self.rest._get_cifs_service_status()
            if status == constants.CIFS_SHARE_MODE:
                self.rest._config_cifs_service()
        elif cifs_status == constants.CIFS_SHARE_MODE:
            self.rest._config_cifs_service()

    def do_setup(self):
        """get token"""
        self.rest.login()

    def create_share(self, share, share_server=None):
        """Create a share"""
        pool_name, share_name, proto = self._get_share_instance_pnp(share)

        share_size = ''.join((str(share['size']), 'GB'))

        # check pool available
        storage_pools = self.rest._get_all_pool()
        pool_info = self._find_pool_info(pool_name, storage_pools)
        if not pool_info:
            msg = f'Failed to find information regarding pool {pool_name}'
            msg = _(msg)
            raise exception.InvalidHost(reason=msg)

        # first create filesystem
        self.rest._create_filesystem(fs_name=share_name,
                                     pool_name=pool_name,
                                     filesystem_quota=share_size)

        share_path = self._generate_share_path(share_name)
        # second create filesystem dir
        self.rest._create_filesystem_dir(share_path)
        # third create nfs or cifs share
        if proto == 'NFS':
            self.rest._create_nfs_share(share_path=share_path)
        else:
            user_name = 'manilanobody'
            user_passwd = 'manilanobody'
            group_name = 'manilanobody'
            ret = self._ensure_user(user_name, user_passwd, group_name)
            if not ret:
                self.rest._delete_filesystem(share_name)
                raise exception.MacrosanBackendExeption(
                    reason=(_(
                        'Failed to create share %(share)s.Reason:'
                        'username %(user_name)s error.')
                        % {'share': share_name, 'user_name': user_name}))

            rw_list = [user_name]
            rw_list_type = ['0']
            self.rest._create_cifs_share(share_name=share_name,
                                         share_path=share_path,
                                         rw_list=rw_list,
                                         rw_list_type=rw_list_type)

        location = self._get_location_path(share_path, share_name, proto)
        return location

    def delete_share(self, share, share_server=None):
        """Delete a share."""
        pool, share_name, proto = self._get_share_instance_pnp(share)
        share_path = self._generate_share_path(share_name)

        backend_share = self._get_share(share_path, proto)
        if not backend_share:
            LOG.error(f'Share {share_name} not found.')
            filesystem = self.rest._get_filesystem(share_name)
            if filesystem:
                self.rest._delete_filesystem(share_name)
        else:
            if proto == 'NFS':
                self.rest._delete_nfs_share(share_path)
            else:
                self.rest._delete_cifs_share(share_name, share_path)
            self.rest._delete_filesystem(share_name)

    def extend_share(self, share, new_size, share_server=None):
        """Extend share"""
        pool, share_name, proto = self._get_share_instance_pnp(share)
        share_path = self._generate_share_path(share_name)
        backend_share = self._get_share(share_path, proto)
        if not backend_share:
            msg = f"Can't find the share by share name: {share_name}."
            msg = _(msg)
            LOG.error(msg)
            raise exception.ShareResourceNotFound(share_id=share['id'])

        # storage size logic already in manila/share/api.py extend func
        # size param need unit
        new_size = ''.join((str(new_size), 'GB'))
        self.rest._update_share_size(share_name, new_size)

    def shrink_share(self, share, new_size, share_server=None):
        """Shrink share"""
        pool, share_name, proto = self._get_share_instance_pnp(share)
        share_path = self._generate_share_path(share_name)
        backend_share = self._get_share(share_path, proto)
        if not backend_share:
            msg = f"Can't find the share by share name: {share_name}."
            msg = _(msg)
            LOG.error(msg)
            raise exception.ShareResourceNotFound(share_id=share['id'])

        filesystem_info = self.rest._get_filesystem(share_name)
        used_size = self._unit_convert_toGB(filesystem_info['usedCapacity'])
        if new_size <= used_size:
            raise exception.ShareShrinkingPossibleDataLoss(
                share_id=share['id'])
        # storage size logic already in manila/share/api.py shrink func
        new_size = ''.join((str(new_size), 'GB'))
        self.rest._update_share_size(share_name, new_size)

    def ensure_share(self, share, share_server=None):
        """Enusre that share is exported"""
        pool, share_name, proto = self._get_share_instance_pnp(share)
        share_path = self._generate_share_path(share_name)
        backend_share = self._get_share(share_path, proto)
        if not backend_share:
            raise exception.ShareResourceNotFound(share_id=share['id'])

        location = self._get_location_path(share_path, share_name, proto)
        return [location]

    def _allow_access(self, share, access, share_server=None):
        """Allow access to the share."""
        pool, share_name, proto = self._get_share_instance_pnp(share)
        share_path = self._generate_share_path(share_name)
        access_level = access['access_level']
        share_id = share['id']
        if access_level not in ('rw', 'ro'):
            raise exception.InvalidShareAccess(
                reason=(_('Unsupported access level: %s.') % access_level))
        if proto == 'NFS':
            self._allow_nfs_access(share_path, share_name, access, share_id)
        elif proto == 'CIFS':
            self._allow_cifs_access(share_path, share_name, access, share_id)

    def _allow_nfs_access(self, share_path, share_name, access, share_id):
        """Allow nfs access."""
        access_type = access['access_type']
        access_to = access['access_to']
        access_level = access['access_level']
        # Only use 'ip',
        # input "*" replace all, or ip 172.0.1.11 ,
        # or  ip network segment 172.0.1.11/255.255.0.0  172.0.1.11/16
        if access_type != 'ip':
            message = (_('NFS shares only allow IP access types. '
                         'access_type: %(access_type)s') %
                       {'access_type': access_type})
            raise exception.InvalidShareAccess(reason=message)
        backend_share = self.rest._get_nfs_share(share_path)
        if not backend_share:
            msg = (_("Can't find the share by share name: %s.")
                   % share_name)
            LOG.error(msg)
            raise exception.ShareResourceNotFound(share_id=share_id)

        if access_to == '0.0.0.0/0':
            access_to = '*'
        share_client = self.rest._get_access_from_nfs_share(share_path,
                                                            access_to)

        if share_client:
            if access_level != share_client['accessRight']:
                self.rest._change_nfs_access_rest(share_path,
                                                  access_to,
                                                  access_level)
        else:
            self.rest._allow_nfs_access_rest(share_path,
                                             access_to,
                                             access_level)

    def _allow_cifs_access(self, share_path, share_name, access, share_id):
        """Allow cifs access."""
        access_type = access['access_type']
        access_to = access['access_to']
        access_level = access['access_level']
        if access_type != 'user':
            message = _('Only user access type is '
                        'allowed for CIFS shares.')
            raise exception.InvalidShareAccess(reason=message)

        backend_share = self.rest._get_cifs_share(share_path)
        if not backend_share:
            msg = (_("Can't find the share by share name: %s.")
                   % share_name)
            LOG.error(msg)
            raise exception.ShareResourceNotFound(share_id=share_id)

        share_client = self.rest._get_access_from_cifs_share(share_path,
                                                             access_to)
        if share_client:
            if access_level != share_client['accessRight']:
                self.rest._change_cifs_access_rest(share_path,
                                                   access_to,
                                                   access_level,
                                                   share_client['ugType'])
        else:
            self.rest._allow_cifs_access_rest(share_path,
                                              access_to,
                                              access_level)

    def _deny_access(self, share, access, share_server=None):
        """Deny access to the share."""
        pool, share_name, proto = self._get_share_instance_pnp(share)
        share_path = self._generate_share_path(share_name)

        if proto == 'NFS':
            self._deny_nfs_access(share_path, share_name, access)
        else:
            self._deny_cifs_access(share_path, share_name, access)

    def _deny_nfs_access(self, share_path, share_name, access):
        """Deny nfs access."""
        access_type = access['access_type']
        access_to = access['access_to']
        if access_type != 'ip':
            LOG.error('Only IP access types are allowed '
                      'for NFS shares.')
            return
        if access_to == '0.0.0.0/0':
            access_to = '*'
        share_client = self.rest._get_access_from_nfs_share(share_path,
                                                            access_to)
        if not share_client:
            LOG.error(f'Could not list the share access for share '
                      f'{share_name}')
            return
        self.rest._delete_nfs_access_rest(share_path, access_to)

    def _deny_cifs_access(self, share_path, share_name, access):
        """Deny cifs access."""
        access_type = access['access_type']
        access_to = access['access_to']
        if access_type != 'user':
            LOG.error('Only USER access types are allowed '
                      'for CIFS shares.')
            return
        share_client = self.rest._get_access_from_cifs_share(share_path,
                                                             access_to)
        if not share_client:
            LOG.error(f'Could not list the share access for share '
                      f'{share_name}')
            return
        self.rest._delete_cifs_access_rest(share_path,
                                           share_client['ugName'],
                                           share_client['ugType'])

    def _clear_access(self, share, share_server=None):
        """Remove all access rules of the share"""
        pool, share_name, proto = self._get_share_instance_pnp(share)
        share_path = self._generate_share_path(share_name)
        access_list = self._get_all_access_from_share(share_path, proto)
        if not access_list:
            LOG.error(f'Could not list the share access for share '
                      f'{share_name}')
            return

        if proto == 'NFS':
            for share_access in access_list:
                if share_access['access_to'] == '172.0.0.2':
                    continue
                self.rest._delete_nfs_access_rest(share_path,
                                                  share_access['access_to'])
        elif proto == 'CIFS':
            for share_access in access_list:
                if (share_access['access_to'] == 'manilanobody'
                        and share_access['ugType'] == '0'):
                    continue
                self.rest._delete_cifs_access_rest(share_path,
                                                   share_access['access_to'],
                                                   share_access['ugType'])

    def update_access(self, share, access_rules, add_rules,
                      delete_rules, share_server=None):
        """Update access rules list."""
        access_updates = {}
        if not (add_rules or delete_rules):
            self._clear_access(share, share_server)
            for access_rule in access_rules:
                try:
                    self._allow_access(share, access_rule, share_server)
                except exception.InvalidShareAccess as e:
                    msg = f'Failed to allow {access_rule["access_level"]} ' \
                          f'access to {access_rule["access_to"]}, reason {e}'
                    msg = _(msg)
                    LOG.error(msg)
                    access_updates.update(
                        {access_rule['access_id']: {'state': 'error'}})
        else:
            for access_rule in delete_rules:
                self._deny_access(share, access_rule, share_server)
            for access_rule in add_rules:
                try:
                    self._allow_access(share, access_rule, share_server)
                except exception.InvalidShareAccess as e:
                    msg = f'Failed to allow {access_rule["access_level"]} ' \
                          f'access to {access_rule["access_to"]}, reason {e}'
                    msg = _(msg)
                    LOG.error(msg)
                    access_updates.update(
                        {access_rule['access_id']: {'state': 'error'}})
        return access_updates

    def _get_all_access_from_share(self, share_path, share_proto):
        access_list = []
        if share_proto == 'NFS':
            access_list = self.rest._get_all_nfs_access_rest(share_path)
        elif share_proto == 'CIFS':
            access_list = self.rest._get_all_cifs_access_rest(share_path)
        return access_list

    def _ensure_user(self, user_name, user_passwd, group_name):
        ret_user = self.rest._query_user(user_name)
        if ret_user == constants.USER_NOT_EXIST:
            ret_group = self.rest._query_group(group_name)
            if ret_group not in [constants.GROUP_NOT_EXIST,
                                 constants.GROUP_EXIST]:
                msg = f'Failed to use group {group_name}'
                msg = _(msg)
                raise exception.InvalidInput(reason=msg)
            elif ret_group == constants.GROUP_NOT_EXIST:
                self.rest._add_localgroup(group_name)
            self.rest._add_localuser(user_name, user_passwd, group_name)
            return True
        elif ret_user == constants.USER_EXIST:
            return True
        else:
            return False

    def update_share_stats(self, dict_data):
        """Update pools info"""

        result = self.rest._get_all_pool()
        dict_data["pools"] = []
        for pool_name in self.pools:

            pool_capacity = self._get_pool_capacity(pool_name, result)

            if pool_capacity:
                pool = {
                    'pool_name': pool_name,
                    'total_capacity_gb': pool_capacity['totalcapacity'],
                    'free_capacity_gb': pool_capacity['freecapacity'],
                    'allocated_capacity_gb':
                        pool_capacity['allocatedcapacity'],
                    'reserved_percentage': 0,
                    'reserved_snapshot_percentage': 0,
                    'reserved_share_extend_percentage': 0,
                    'dedupe': False,
                    'compression': False,
                    'qos': False,
                    'thin_provisioning': False,
                    'snapshot_support': self.snapshot_support,
                    'create_share_from_snapshot_support':
                        self.snapshot_support,
                }

                dict_data["pools"].append(pool)

        if not dict_data['pools']:
            msg = _("StoragePool is None")
            LOG.error(msg)
            raise exception.InvalidInput(reason=msg)

    def _get_pool_capacity(self, pool_name, result):
        """Get total,allocated,free capacity of the pools"""
        pool_info = self._find_pool_info(pool_name, result)

        if pool_info:
            total_capacity = int(self._unit_convert_toGB(
                pool_info['totalcapacity']))
            free_capacity = int(self._unit_convert_toGB(
                pool_info['freecapacity']))
            allocated_capacity = int(self._unit_convert_toGB(
                pool_info['allocatedcapacity']))

            pool_info['totalcapacity'] = total_capacity
            pool_info['freecapacity'] = free_capacity
            pool_info['allocatedcapacity'] = allocated_capacity

        return pool_info

    def _unit_convert_toGB(self, capacity):
        """Convert unit to GB"""
        capacity = capacity.upper()

        try:
            # get unit char array and use join connect to string
            unit = re.findall(r'[A-Z]', capacity)
            unit = ''.join(unit)
        except BaseException:
            unit = ''
        # get capacity size,unit is GB
        capacity = capacity.replace(unit, '')
        capacity = float(capacity)
        if unit in ['B', '']:
            capacity = capacity / units.Gi
        elif unit in ['K', 'KB']:
            capacity = capacity / units.Mi
        elif unit in ['M', 'MB']:
            capacity = capacity / units.Ki
        elif unit in ['G', 'GB']:
            capacity = capacity
        elif unit in ['T', 'TB']:
            capacity = capacity * units.Ki
        elif unit in ['E', 'EB']:
            capacity = capacity * units.Mi

        capacity = '%.0f' % capacity

        return float(capacity)

    def _generate_share_name(self, share):
        share_name = 'manila_%s' % share['id']
        return self._format_name(share_name)

    def _format_name(self, name):
        """format name to meet the backend requirements"""
        name = name[0: 31]
        name = name.replace('-', '_')
        return name

    def _generate_share_path(self, share_name):
        """add '/' as path"""
        share_path = r'/%(path)s/%(dirName)s' % {
            'path': share_name.replace("-", "_"),
            'dirName': share_name.replace("-", "_")
        }
        return share_path

    def _get_location_path(self, share_path, share_name, share_proto, ip=None):
        location = None
        if ip is None:
            ip = self.configuration.macrosan_nas_ip
        share_proto = share_proto.upper()
        if share_proto == 'NFS':
            location = f'{ip}:{share_path}'
        elif share_proto == 'CIFS':
            location = f'\\\\{ip}\\{share_name}'
        return location

    def _get_share_instance_pnp(self, share_instance):

        proto = share_instance['share_proto'].upper()
        share_name = self._generate_share_name(share_instance)
        pool = share_utils.extract_host(share_instance['host'], level='pool')
        if not pool:
            msg = _("Pool doesn't exist in host field.")
            raise exception.InvalidHost(reason=msg)

        if proto != 'NFS' and proto != 'CIFS':
            msg = f'Share protocol {proto} is not supported.'
            msg = _(msg)
            raise exception.MacrosanBackendExeption(reason=msg)

        return pool, share_name, proto

    def _get_share(self, share_path, proto):
        return (self.rest._get_nfs_share(share_path)
                if proto == 'NFS'
                else self.rest._get_cifs_share(share_path))

    def _find_pool_info(self, pool_name, result):
        if pool_name is None:
            return
        pool_info = {}
        pool_name = pool_name.strip()

        for item in result.get('data', []):
            if pool_name == item['name']:
                pool_info['name'] = item['name']
                pool_info['totalcapacity'] = item['size']
                pool_info['allocatedcapacity'] = item['allocated']
                pool_info['freecapacity'] = item['free']
                pool_info['health'] = item['health']
                pool_info['rw'] = item['rwStatus']
                break

        return pool_info



# Copyright 2016 Mirantis, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.
"""Some useful fakes."""

from manila.tests.db import fakes as db_fakes

FAKE_VSCTL_LIST_INTERFACES_X = (
    'fake stuff\n'
    'foo not_a_veth something_fake bar\n'
    'foo veth11b2c34 something_fake bar\n'
    'foo veth25f6g7h manila-container="fake1" bar\n'
    'foo veth3jd83j7 manila-container="my_container" bar\n'
    'foo veth4i9j10k manila-container="fake2" bar\n'
    'more fake stuff\n'
)

FAKE_VSCTL_LIST_INTERFACES = (
    'fake stuff\n'
    'foo not_a_veth something_fake bar\n'
    'foo veth11b2c34 something_fake bar\n'
    'foo veth25f6g7h manila-container="fake1" bar\n'
    'foo veth3jd83j7 manila-container="manila_my_container" bar\n'
    'foo veth4i9j10k manila-container="fake2" bar\n'
    'more fake stuff\n'
)

FAKE_VSCTL_LIST_INTERFACE_1 = (
    'fake stuff\n'
    'foo veth11b2c34 something_fake bar\n'
    'more fake stuff\n'
)

FAKE_VSCTL_LIST_INTERFACE_2 = (
    'fake stuff\n'
    'foo veth25f6g7h manila-container="fake1" bar\n'
    'more fake stuff\n'
)

FAKE_VSCTL_LIST_INTERFACE_3_X = (
    'fake stuff\n'
    'foo veth3jd83j7 manila-container="my_container" bar\n'
    'more fake stuff\n'
)

FAKE_VSCTL_LIST_INTERFACE_3 = (
    'fake stuff\n'
    'foo veth3jd83j7 manila-container="manila_my_container" bar\n'
    'more fake stuff\n'
)

FAKE_VSCTL_LIST_INTERFACE_4 = (
    'fake stuff\n'
    'foo veth4i9j10k manila-container="fake2" bar\n'
    'more fake stuff\n'
)

FAKE_IP_LINK_SHOW = (
    ('1: lo: <LOOPBACK,UP,LOWER_UP> mtu 65536 qdisc noqueue state UNKNOWN '
     'mode DEFAULT group default qlen 1000\\    link/loopback '
     '00:00:00:00:00:00 brd 00:00:00:00:00:00\n'
     '13: eth0@if16: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc noqueue '
     'state UP mode DEFAULT group default \\    link/ether 02:42:ac:15:00:02 '
     'brd ff:ff:ff:ff:ff:ff\n'
     '15: eth1@if14: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc noqueue '
     'state UP mode DEFAULT group default \\    link/ether 02:42:ac:14:00:02 '
     'brd ff:ff:ff:ff:ff:ff\n', '')
)

FAKE_IP_LINK_SHOW_MASTER = (
    ('16: fake_veth@if14: <BROADCAST,MULTICAST,UP,LOWER_UP> mtu 1500 qdisc '
     'noqueue master br-a7d71c3e77c2 state UP mode DEFAULT group default\n'
     '    link/ether 4a:10:0c:f2:d2:2c brd ff:ff:ff:ff:ff:ff link-netnsid 0\n',
     '')
)

FAKE_IP_ADDR_SHOW = (
    [('283: eth0    inet 192.168.144.19/24 brd 192.168.144.255 scope global '
      'eth0\\       valid_lft forever preferred_lft forever', ''),
     ('287: eth1    inet 10.0.0.131/8 brd 8.255.255.255 scope global eth1\\   '
      '    valid_lft forever preferred_lft forever', '')]
)

FAKE_DOCKER_INSPECT_NETWORKS = (
    ('{"fake_docker_network_0":{"IPAMConfig":{},"Links":null,"Aliases":'
     '["dab16d2703dc"],"NetworkID":'
     '"cf8c7cb5cecda1ef8240921d5d09e2a1bf9e308a0261459f5a69114cd4e6283c",'
     '"EndpointID":'
     '"312a035f32be713c7b56093dde2beec950785ddeb29c9bd18018d43ffd4f64bd",'
     '"Gateway":"10.10.10.1","IPAddress":"10.10.10.10","IPPrefixLen":24,'
     '"IPv6Gateway":"","GlobalIPv6Address":"","GlobalIPv6PrefixLen":0,'
     '"MacAddress":"10:10:10:10:10:10","DriverOpts":{}},'
     '"fake_docker_network_1":{"IPAMConfig":{},"Links":null,"Aliases":'
     '["dab16d2703dc"],"NetworkID":'
     '"e978d91d70c30695557018c8847a551267e99c083063391c07dc9a730bfef9dc",'
     '"EndpointID":'
     '"8e34044764cd52b9d092ac66af8fb7130cdd423b521c3bf6e57b8095f6f0a085",'
     '"Gateway":"20.20.20.1","IPAddress":"20.20.20.20","IPPrefixLen":24,'
     '"IPv6Gateway":"","GlobalIPv6Address":"","GlobalIPv6PrefixLen":0,'
     '"MacAddress":"20:20:20:20:20:20","DriverOpts":{}}}', '')
)


def fake_share(**kwargs):
    share = {
        'id': 'fakeid',
        'share_id': 'fakeshareid',
        'name': 'fakename',
        'size': 1,
        'share_proto': 'NFS',
        'host': 'host@backend#vg',
        'export_location': '127.0.0.1:/mnt/nfs/volume-00002',
    }
    share.update(kwargs)
    return db_fakes.FakeModel(share)


def fake_share_instances(**kwargs):
    share_instances = {
        'id': 'fakeid',
        'share_id': 'fakeshareid',
        'host': 'host@backend#vg',
        'export_location': '127.0.0.1:/mnt/nfs/volume-00002',
    }
    share_instances.update(kwargs)
    return [db_fakes.FakeModel(share_instances)]


def fake_access(**kwargs):
    access = {
        'id': 'fakeaccid',
        'access_type': 'ip',
        'access_to': '10.0.0.2',
        'access_level': 'rw',
        'state': 'active',
    }
    access.update(kwargs)
    return db_fakes.FakeModel(access)


def fake_network(**kwargs):
    allocations = db_fakes.FakeModel({'id': 'fake_allocation_id',
                                      'ip_address': '127.0.0.0.1',
                                      'mac_address': 'fe:16:3e:61:e0:58'})
    network = {
        'id': 'fake_network_id',
        'server_id': 'fake_server_id',
        'network_allocations': [allocations],
        'neutron_net_id': 'fake_net',
        'neutron_subnet_id': 'fake_subnet'
    }
    network.update(kwargs)
    return [db_fakes.FakeModel(network)]


def fake_network_with_security_services(**kwargs):
    allocations = db_fakes.FakeModel({'id': 'fake_allocation_id',
                                      'ip_address': '127.0.0.0.1',
                                      'mac_address': 'fe:16:3e:61:e0:58'})
    security_services = db_fakes.FakeModel({'status': 'fake_status',
                                            'id': 'fake_security_service_id',
                                            'project_id': 'fake_project_id',
                                            'type': 'fake_type',
                                            'name': 'fake_name'})
    network = {
        'id': 'fake_network_id',
        'server_id': 'fake_server_id',
        'network_allocations': [allocations],
        'neutron_net_id': 'fake_net',
        'neutron_subnet_id': 'fake_subnet',
        'security_services': [security_services],
    }
    network.update(kwargs)
    return [db_fakes.FakeModel(network)]


def fake_share_server(**kwargs):
    share_server = {
        'id': 'fake'
    }
    share_server.update(kwargs)
    return db_fakes.FakeModel(share_server)


def fake_identifier():
    return '7cf7c200-d3af-4e05-b87e-9167c95dfcad'


def fake_share_no_export_location(**kwargs):
    share = {
        'share_id': 'fakeshareid',
    }
    share.update(kwargs)
    return db_fakes.FakeModel(share)


def fake_current_network_allocations():
    current_network_allocations = {
        'subnets': [
            {
                'network_allocations': [
                    {
                        'id': 'fake_id_current',
                        'ip_address': '192.168.144.100',
                    }
                ]
            }
        ]
    }

    return current_network_allocations


def fake_new_network_allocations():
    new_network_allocations = {
        'network_allocations': [
            {
                'id': 'fake_id_new',
                'ip_address': '10.0.0.100',
            }
        ]
    }

    return new_network_allocations



# Copyright (c) 2016 Mirantis, Inc.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

import re
import uuid

from oslo_log import log
from oslo_serialization import jsonutils
from oslo_utils import excutils

from manila import exception
from manila.i18n import _
from manila.share import driver


LOG = log.getLogger(__name__)


class DockerExecHelper(driver.ExecuteMixin):
    def __init__(self, *args, **kwargs):
        self.configuration = kwargs.pop("configuration", None)
        super(DockerExecHelper, self).__init__(*args, **kwargs)
        self.init_execute_mixin()

    def create_container(self, name=None):
        name = name or "".join(["manila_cifs_docker_container",
                                str(uuid.uuid1()).replace("-", "_")])
        image_name = self.configuration.container_image_name
        LOG.debug("Starting container from image %s.", image_name)
        # (aovchinnikov): --privileged is required for both samba and
        # nfs-ganesha to actually allow access to shared folders.
        #
        # (aovchinnikov): To actually make docker container mount a
        # logical volume created after container start-up to some location
        # inside it, we must share entire /dev with it. While seemingly
        # dangerous it is not and moreover this is apparently the only sane
        # way to do it. The reason is when a logical volume gets created
        # several new things appear in /dev: a new /dev/dm-X and a symlink
        # in /dev/volume_group_name pointing to /dev/dm-X. But to be able
        # to interact with /dev/dm-X, it must be already present inside
        # the container's /dev i.e. it must have been -v shared during
        # container start-up. So we should either precreate an unknown
        # number of /dev/dm-Xs (one per LV), share them all and hope
        # for the best or share the entire /dev and hope for the best.
        #
        # The risk of allowing a container having access to entire host's
        # /dev is not as big as it seems: as long as actual share providers
        # are invulnerable this does not pose any extra risks. If, however,
        # share providers contain vulnerabilities then the driver does not
        # provide any more possibilities for an exploitation than other
        # first-party drivers.
        path = "{0}:/shares".format(
            self.configuration.container_volume_mount_path)
        cmd = ["docker", "container", "create", "--name=%s" % name,
               "--privileged", "-v", "/dev:/dev", "-v", path, image_name]
        try:
            result = self._inner_execute(cmd)
        except (exception.ProcessExecutionError, OSError):
            raise exception.ShareBackendException(
                msg="Container %s failed to be created." % name)

        self.disconnect_network("bridge", name)
        LOG.info("A container has been successfully created! Its id is %s.",
                 result[0].rstrip("\n"))

    def start_container(self, name):
        cmd = ["docker", "container", "start", name]

        try:
            self._inner_execute(cmd)
        except (exception.ProcessExecutionError, OSError):
            raise exception.ShareBackendException(
                msg="Container %s has failed to start." % name)

        LOG.info("Container %s successfully started!", name)

    def stop_container(self, name):
        LOG.debug("Stopping container %s.", name)
        try:
            self._inner_execute(["docker", "stop", name])
        except (exception.ProcessExecutionError, OSError):
            raise exception.ShareBackendException(
                msg="Container %s has failed to stop properly." % name)
        LOG.info("Container %s is successfully stopped.", name)

    def execute(self, name=None, cmd=None, ignore_errors=False):
        if name is None:
            raise exception.ManilaException(_("Container name not specified."))
        if cmd is None or (type(cmd) is not list):
            raise exception.ManilaException(_("Missing or malformed command."))
        LOG.debug("Executing inside a container %s.", name)
        cmd = ["docker", "exec", "-i", name] + cmd
        result = self._inner_execute(cmd, ignore_errors=ignore_errors)
        return result

    def _inner_execute(self, cmd, ignore_errors=False):
        LOG.debug("Executing command: %s.", " ".join(cmd))
        try:
            result = self._execute(*cmd, run_as_root=True)
        except (exception.ProcessExecutionError, OSError) as e:
            with excutils.save_and_reraise_exception(
                    reraise=not ignore_errors):
                LOG.warning("Failed to run command %(cmd)s due to "
                            "%(reason)s.", {'cmd': cmd, 'reason': e})
        else:
            LOG.debug("Execution result: %s.", result)
            return result

    def fetch_container_addresses(self, name, address_family="inet6"):
        addresses = []
        interfaces = self.fetch_container_interfaces(name)

        for interface in interfaces:
            result = self.execute(
                name,
                ["ip", "-oneline",
                 "-family", address_family,
                 "address", "show", "scope", "global", "dev", interface],
            )
            address_w_prefix = result[0].split()[3]
            addresses.append(address_w_prefix.split("/")[0])

        return addresses

    def fetch_container_interfaces(self, name):
        interfaces = []
        links = self.execute(name, ["ip", "-o", "link", "show"])
        links = links[0].rstrip().split("\n")
        links = [link for link in links if link.split()[1].startswith("eth")]

        for link in links:
            interface = re.search(" (.+?)@", link).group(1)
            interfaces.append(interface)

        return interfaces

    def rename_container(self, name, new_name):
        veth_names = self.get_container_veths(name)
        if not veth_names:
            raise exception.ManilaException(
                _("Could not find OVS information related to "
                  "container %s.") % name)

        try:
            self._inner_execute(["docker", "rename", name, new_name])
        except (exception.ProcessExecutionError, OSError):
            raise exception.ShareBackendException(
                msg="Could not rename container %s." % name)

        for veth_name in veth_names:
            cmd = ["ovs-vsctl", "set", "interface", veth_name,
                   "external-ids:manila-container=%s" % new_name]
            try:
                self._inner_execute(cmd)
            except (exception.ProcessExecutionError, OSError):
                try:
                    self._inner_execute(["docker", "rename", new_name, name])
                except (exception.ProcessExecutionError, OSError):
                    msg = _("Could not rename back container %s.") % name
                    LOG.exception(msg)
                raise exception.ShareBackendException(
                    msg="Could not update OVS information %s." % name)

        LOG.info("Container %s has been successfully renamed.", name)

    def container_exists(self, name):

        result = self._execute("docker", "ps", "--no-trunc",
                               "--format='{{.Names}}'", run_as_root=True)[0]
        for line in result.split('\n'):
            if name == line.strip("'"):
                return True
        return False

    def create_network(self, network_name):
        cmd = ["docker", "network", "create", network_name]
        LOG.debug("Creating the %s Docker network.", network_name)

        try:
            result = self._inner_execute(cmd)
        except (exception.ProcessExecutionError, OSError):
            raise exception.ShareBackendException(
                msg="Docker network %s could not be created." % network_name)

        LOG.info("The Docker network has been successfully created! Its id is "
                 "%s.", result[0].rstrip("\n"))

    def remove_network(self, network_name):
        cmd = ["docker", "network", "remove", network_name]
        LOG.debug("Removing the %s Docker network.", network_name)

        try:
            result = self._inner_execute(cmd)
        except (exception.ProcessExecutionError, OSError):
            raise exception.ShareBackendException(
                msg="Docker network %s could not be removed. One or more "
                    "containers are probably still using it." % network_name)

        LOG.info("The %s Docker network has been successfully removed!",
                 result[0].rstrip("\n"))

    def connect_network(self, network_name, container_name):
        cmd = ["docker", "network", "connect", network_name, container_name]

        try:
            self._inner_execute(cmd)
        except (exception.ProcessExecutionError, OSError):
            raise exception.ShareBackendException(
                msg="Could not connect the Docker network %s to container %s."
                    % (network_name, container_name))

        LOG.info("Docker network %s has been successfully connected to "
                 "container %s!", network_name, container_name)

    def disconnect_network(self, network_name, container_name):
        cmd = ["docker", "network", "disconnect", network_name, container_name]

        try:
            self._inner_execute(cmd)
        except (exception.ProcessExecutionError, OSError):
            raise exception.ShareBackendException(
                msg="Could not disconnect the Docker network %s from "
                    "container %s." % (network_name, container_name))

        LOG.debug("Docker network %s has been successfully disconnected from "
                  "container %s!", network_name, container_name)

    def get_container_networks(self, container_name):
        cmd = ["docker", "container", "inspect", "-f",
               "'{{json .NetworkSettings.Networks}}'", container_name]

        try:
            result = self._inner_execute(cmd)
        except (exception.ProcessExecutionError, OSError):
            raise exception.ShareBackendException(
                msg="Could not find any networks associated with the %s "
                    "container." % container_name)

        # NOTE(ecsantos): The stdout from _inner_execute comes with extra
        # single quotes.
        networks = list(jsonutils.loads(result[0].strip("\n'")))
        return networks

    def get_container_veths(self, container_name):
        veths = []
        cmd = ["bash", "-c", "cat /sys/class/net/eth*/iflink"]
        eths_iflinks = self.execute(container_name, cmd)

        for eth_iflink in eths_iflinks[0].rstrip().split("\n"):
            veth = self._execute("bash", "-c", "grep -l %s "
                                 "/sys/class/net/veth*/ifindex" % eth_iflink)
            veth = re.search("t/(.+?)/i", veth[0]).group(1)
            veths.append(veth)

        return veths

    def get_network_bridge(self, network_name):
        cmd = ["docker", "network", "inspect", "-f", "{{.Id}}", network_name]

        try:
            network_id = self._inner_execute(cmd)
        except (exception.ProcessExecutionError, OSError):
            raise exception.ShareBackendException(
                msg="Could not find the ID of the %s Docker network."
                    % network_name)

        # The name of the bridge associated with a given Docker network is
        # always "br-" followed by the first 12 digits of that network's ID.
        return "br-" + network_id[0][0:12]

    def get_veth_from_bridge(self, bridge):
        veth = self._execute("ip", "link", "show", "master", bridge)
        veth = re.search(" (.+?)@", veth[0]).group(1)
        return veth



# Copyright (c) 2022 MacroSAN Technologies Co., Ltd.
# All Rights Reserved.
#
#    Licensed under the Apache License, Version 2.0 (the "License"); you may
#    not use this file except in compliance with the License. You may obtain
#    a copy of the License at
#
#         http://www.apache.org/licenses/LICENSE-2.0
#
#    Unless required by applicable law or agreed to in writing, software
#    distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#    WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#    License for the specific language governing permissions and limitations
#    under the License.

"""
Share driver test for Macrosan Storage Array.
"""

import ddt
import requests

from oslo_config import cfg
from unittest import mock

from manila import context
from manila import exception
from manila import test

from manila.share import configuration
from manila.share import driver
from manila.share.drivers.macrosan import macrosan_constants as constants
from manila.share.drivers.macrosan import macrosan_helper
from manila.share.drivers.macrosan import macrosan_nas
from manila.share.drivers.macrosan import rest_helper
from manila.tests import fake_share

CONF = cfg.CONF


class FakeResponse(object):
    def __init__(self, status, result):
        self.status_code = status
        self.text = 'return message'
        self.response = result

    def json(self):
        return self.response

    def close(self):
        pass


@ddt.ddt
class MacrosanShareDriverTestCase(test.TestCase):

    def setUp(self):
        self.mock_object(macrosan_nas.CONF, '_check_required_opts')
        super(MacrosanShareDriverTestCase, self).setUp()

        def _safe_get(opt):
            return getattr(self.configuration, opt)

        self._context = context.get_admin_context()
        self.configuration = mock.Mock(spec=configuration.Configuration)
        self.configuration.safe_get = mock.Mock(side_effect=_safe_get)
        self.configuration.driver_handles_share_servers = False
        self.configuration.share_backend_name = 'fake_share_backend_name'
        self.configuration.macrosan_nas_http_protocol = 'https'
        self.configuration.macrosan_nas_ip = 'fake_ip'
        self.configuration.macrosan_nas_port = 'fake_port'
        self.configuration.macrosan_nas_username = 'fake_username'
        self.configuration.macrosan_nas_password = 'fake_password'
        self.configuration.macrosan_nas_prefix = 'nas'
        self.configuration.macrosan_share_pools = ['fake_pool']
        self.configuration.macrosan_timeout = 60

        self.configuration.network_config_group = 'fake_network_config_group'
        self.configuration.admin_network_config_group = (
            'fake_admin_network_config_group')
        self.configuration.config_group = 'fake_config_group'
        self.configuration.reserved_share_percentage = 0
        self.configuration.filter_function = None
        self.configuration.goodness_function = None
        self.driver = macrosan_nas.MacrosanNasDriver(
            configuration=self.configuration)
        self.resutl_success_storage_pools = {
            'code': 0,
            'message': 'success',
            'data': [{
                'name': 'fake_pool',
                'size': '1000.0G',
                'allocated': '100G',
                'free': '900G',
                'health': 'ONLINE',
                'rwStatus': 'off'
            }]
        }

    def test_do_setup(self):
        mock_login = self.mock_object(rest_helper.RestHelper, 'login')
        self.driver.do_setup(self._context)
        mock_login.assert_called_once()

    def test_do_setup_login_fail(self):
        mock_login = self.mock_object(
            rest_helper.RestHelper, 'login',
            mock.Mock(
                side_effect=exception.ShareBackendException(
                    msg='fake_exception')))
        self.assertRaises(exception.ShareBackendException,
                          self.driver.do_setup,
                          self._context)
        mock_login.assert_called_once()

    @ddt.data({'nfs_status': constants.NFS_NON_CONFIG,
               'cifs_status': constants.CIFS_NON_CONFIG},
              {'nfs_status': constants.NFS_DISABLED,
               'cifs_status': constants.CIFS_DISABLED},
              {'nfs_status': constants.NFS_ENABLED,
               'cifs_status': constants.CIFS_ENABLED},
              {'nfs_status': constants.NFS_ENABLED,
               'cifs_status': constants.CIFS_SHARE_MODE})
    @ddt.unpack
    def test_check_for_setup_error_non_config(self, nfs_status, cifs_status):
        mock_gnss = self.mock_object(
            rest_helper.RestHelper, '_get_nfs_service_status',
            mock.Mock(return_value={
                "serviceStatus": nfs_status,
                "nfs3Status": constants.NFS_NON_SUPPORTED,
                "nfs4Status": constants.NFS_NON_SUPPORTED
            }))

        mock_cns = self.mock_object(rest_helper.RestHelper,
                                    '_config_nfs_service')
        mock_sns = self.mock_object(rest_helper.RestHelper,
                                    '_start_nfs_service')
        if cifs_status == constants.CIFS_DISABLED:
            mock_gcss = self.mock_object(
                rest_helper.RestHelper, '_get_cifs_service_status',
                mock.Mock(side_effect=[cifs_status,
                                       constants.CIFS_SHARE_MODE]))
        else:
            mock_gcss = self.mock_object(
                rest_helper.RestHelper, '_get_cifs_service_status',
                mock.Mock(return_value=cifs_status))
        mock_ccs = self.mock_object(rest_helper.RestHelper,
                                    '_config_cifs_service')
        mock_scs = self.mock_object(rest_helper.RestHelper,
                                    '_start_cifs_service')
        self.driver.check_for_setup_error()
        if (nfs_status == constants.NFS_NON_CONFIG or
                nfs_status == constants.NFS_DISABLED):
            mock_cns.assert_called_once()
            mock_sns.assert_called_once()
        else:
            mock_cns.assert_called_once()
        mock_gnss.assert_called_once()
        if cifs_status == constants.CIFS_NON_CONFIG:
            mock_gcss.assert_called_once()
            mock_ccs.assert_called_once()
            mock_scs.assert_called_once()
        elif cifs_status == constants.CIFS_DISABLED:
            mock_gcss.assert_called()
            mock_ccs.assert_called_once()
            mock_scs.assert_called_once()
        elif cifs_status == constants.CIFS_SHARE_MODE:
            mock_gcss.assert_called_once()
            mock_ccs.assert_called_once()
        else:
            mock_gcss.assert_called_once()

    def test_check_for_setup_error_nfs_service_error(self):
        mock_gnss = self.mock_object(
            rest_helper.RestHelper, '_get_nfs_service_status',
            mock.Mock(return_value={
                "serviceStatus": constants.NFS_EXCEPTION,
                "nfs3Status": constants.NFS_NON_SUPPORTED,
                "nfs4Status": constants.NFS_NON_SUPPORTED
            }))
        self.assertRaises(exception.MacrosanBackendExeption,
                          self.driver.check_for_setup_error)
        mock_gnss.assert_called_once()

    def test_check_for_setup_error_cifs_service_error(self):
        mock_gnss = self.mock_object(
            rest_helper.RestHelper, '_get_nfs_service_status',
            mock.Mock(return_value={
                "serviceStatus": constants.NFS_ENABLED,
                "nfs3Status": constants.NFS_SUPPORTED,
                "nfs4Status": constants.NFS_SUPPORTED
            }))
        mock_gcss = self.mock_object(
            rest_helper.RestHelper, '_get_cifs_service_status',
            mock.Mock(return_value=constants.CIFS_EXCEPTION))
        self.assertRaises(exception.MacrosanBackendExeption,
                          self.driver.check_for_setup_error)
        mock_gnss.assert_called_once()
        mock_gcss.assert_called_once()

    @ddt.data('nfs', 'cifs')
    def test_create_share(self, share_proto):
        share = fake_share.fake_share(
            share_proto=share_proto, host="fake_host@fake_backend#fake_pool")
        self.mock_object(rest_helper.RestHelper, '_get_all_pool')
        self.mock_object(
            macrosan_helper.MacrosanHelper, '_find_pool_info',
            mock.Mock(return_value={
                "name": "fake_pool",
                "totalcapacity": "10G",
                "allocatedcapacity": "0G",
                "freecapacity": "10G",
                "health": "ONLINE",
                "rw": "off"
            }))
        mock_cf = self.mock_object(rest_helper.RestHelper,
                                   '_create_filesystem')
        mock_cfd = self.mock_object(rest_helper.RestHelper,
                                    '_create_filesystem_dir')
        mock_cns = self.mock_object(rest_helper.RestHelper,
                                    '_create_nfs_share')
        self.mock_object(macrosan_helper.MacrosanHelper,
                         '_ensure_user',
                         mock.Mock(return_value=True))
        mock_ccs = self.mock_object(rest_helper.RestHelper,
                                    '_create_cifs_share')
        self.driver.helper.configuration.macrosan_nas_ip = "172.0.0.1"

        location = self.driver.create_share(self._context, share)
        if share_proto == 'nfs':
            expect_location = r'172.0.0.1:/manila_fakeid/manila_fakeid'
            print('test location:', location)
            self.assertEqual(location, expect_location)
        else:
            expect_location = r'\\172.0.0.1\manila_fakeid'
            self.assertEqual(location, expect_location)
        mock_cf.assert_called_once_with(fs_name='manila_fakeid',
                                        pool_name='fake_pool',
                                        filesystem_quota='1GB')
        mock_cf.assert_called()
        share_path = self.driver.helper._generate_share_path('manila_fakeid')
        mock_cfd.assert_called_once_with(share_path)
        if share_proto == 'nfs':
            mock_cns.assert_called_once_with(share_path=share_path)
        else:
            mock_ccs.assert_called_once()

    def test_create_share_pool_fail(self):
        share = fake_share.fake_share(
            share_proto='nfs', host="fake_host@fake_backend#fake_pool")
        self.mock_object(rest_helper.RestHelper, '_get_all_pool')
        self.mock_object(macrosan_helper.MacrosanHelper, '_find_pool_info',
                         mock.Mock(return_value=None))
        self.mock_object(
            rest_helper.RestHelper, '_create_filesystem')
        self.mock_object(
            rest_helper.RestHelper, '_create_nfs_share')

        self.assertRaises(exception.InvalidHost,
                          self.driver.create_share,
                          self._context,
                          share)

    def test_create_share_user_error(self):
        share = fake_share.fake_share(
            share_proto='cifs', host="fake_host@fake_backend#fake_pool")
        self.mock_object(rest_helper.RestHelper, '_get_all_pool')
        self.mock_object(
            macrosan_helper.MacrosanHelper, '_find_pool_info',
            mock.Mock(return_value={
                "name": "fake_pool",
                "totalcapacity": "10G",
                "allocatedcapacity": "0G",
                "freecapacity": "10G",
                "health": "ONLINE",
                "rw": "off"
            }))
        mock_cf = self.mock_object(rest_helper.RestHelper,
                                   '_create_filesystem')
        mock_cfd = self.mock_object(rest_helper.RestHelper,
                                    '_create_filesystem_dir')
        self.mock_object(macrosan_helper.MacrosanHelper,
                         '_ensure_user',
                         mock.Mock(return_value=False))
        mock_df = self.mock_object(rest_helper.RestHelper,
                                   '_delete_filesystem')
        self.assertRaises(exception.MacrosanBackendExeption,
                          self.driver.create_share,
                          self._context,
                          share)
        mock_cf.assert_called_once()
        share_path = self.driver.helper._generate_share_path('manila_fakeid')
        mock_cfd.assert_called_once_with(share_path)
        mock_df.assert_called_once_with('manila_fakeid')

    @ddt.data('nfs', 'cifs')
    def test_delete_share(self, share_proto):
        share = fake_share.fake_share(
            share_proto=share_proto, host="fake_host@fake_backend#fake_pool")
        expect_share_path = self.driver.helper._generate_share_path(
            'manila_fakeid')

        mock_gns = self.mock_object(rest_helper.RestHelper, '_get_nfs_share',
                                    mock.Mock(return_value={
                                        "path": "fake_path",
                                        "clients": ["client"],
                                        "protocol": "fake_protocol"
                                    }))
        mock_dns = self.mock_object(
            rest_helper.RestHelper, '_delete_nfs_share')
        mock_gcs = self.mock_object(rest_helper.RestHelper, '_get_cifs_share',
                                    mock.Mock(return_value={
                                        "path": "fake_path",
                                        "cifsname": "fake_cifsname",
                                        "protocol": "fake_protocol",
                                        "roList": ["fake_ro"],
                                        "rwList": ["fake_rw"],
                                        "allowList": ["fake_allow"],
                                        "denyList": ["fake_deny"]
                                    }))
        mock_dcs = self.mock_object(rest_helper.RestHelper,
                                    '_delete_cifs_share')
        mock_df = self.mock_object(rest_helper.RestHelper,
                                   '_delete_filesystem')
        self.driver.delete_share(self._context, share)

        if share_proto == "nfs":
            mock_gns.assert_called_once_with(expect_share_path)
            mock_dns.assert_called_once_with(expect_share_path)
        else:
            mock_gcs.assert_called_once_with(expect_share_path)
            mock_dcs.assert_called_once_with('manila_fakeid',
                                             expect_share_path)
        mock_df.assert_called_once_with('manila_fakeid')

    @ddt.data('nfs', 'cifs')
    def test_delete_share_not_exist(self, share_proto):
        share = fake_share.fake_share(share_proto=share_proto,
                                      host="fake_host@fake_backend#fake_pool")
        expect_share_path = self.driver.helper._generate_share_path(
            'manila_fakeid')
        mock_gns = self.mock_object(rest_helper.RestHelper, '_get_nfs_share',
                                    mock.Mock(return_value=None))
        mock_gf = self.mock_object(rest_helper.RestHelper, '_get_filesystem',
                                   mock.Mock(return_value={
                                       "name": "fake_name",
                                       "poolName": "fake_pool",
                                       "quotaStatus": "1GB"
                                   }))
        mock_gcs = self.mock_object(rest_helper.RestHelper, '_get_cifs_share',
                                    mock.Mock(return_value=None))
        mock_df = self.mock_object(rest_helper.RestHelper,
                                   '_delete_filesystem')
        self.driver.delete_share(self._context, share)

        if share_proto == 'nfs':
            mock_gns.assert_called_once_with(expect_share_path)
        else:
            mock_gcs.assert_called_once_with(expect_share_path)

        mock_gf.assert_called_once_with('manila_fakeid')
        mock_df.assert_called_once_with('manila_fakeid')

    @ddt.data('nfs', 'cifs')
    def test_extend_share(self, share_proto):
        share = fake_share.fake_share(share_proto=share_proto,
                                      host="fake_host@fake_backend#fake_pool")
        expect_share_path = self.driver.helper._generate_share_path(
            'manila_fakeid')
        mock_gns = self.mock_object(rest_helper.RestHelper, '_get_nfs_share',
                                    mock.Mock(return_value={
                                        "path": "fake_path",
                                        "clients": ["client"],
                                        "protocol": "fake_protocol"
                                    }))
        mock_gcs = self.mock_object(rest_helper.RestHelper, '_get_cifs_share',
                                    mock.Mock(return_value={
                                        "path": "fake_path",
                                        "cifsname": "fake_cifsname",
                                        "protocol": "fake_protocol",
                                        "roList": ["fake_ro"],
                                        "rwList": ["fake_rw"],
                                        "allowList": ["fake_allow"],
                                        "denyList": ["fake_deny"]
                                    }))
        mock_uss = self.mock_object(rest_helper.RestHelper,
                                    '_update_share_size')
        self.driver.extend_share(share, 2)

        if share_proto == 'nfs':
            mock_gns.assert_called_once_with(expect_share_path)
        else:
            mock_gcs.assert_called_once_with(expect_share_path)

        mock_uss.assert_called_once_with('manila_fakeid', '2GB')

    def test_extend_share_not_exist(self):
        share = fake_share.fake_share(share_proto='nfs',
                                      size=1,
                                      host="fake_host@fake_backend#fake_pool")
        expect_share_path = self.driver.helper._generate_share_path(
            'manila_fakeid')
        mock_gns = self.mock_object(rest_helper.RestHelper,
                                    '_get_nfs_share',
                                    mock.Mock(return_value=None))
        self.assertRaises(exception.ShareResourceNotFound,
                          self.driver.extend_share,
                          share,
                          2)

        mock_gns.assert_called_once_with(expect_share_path)

    @ddt.data('nfs', 'cifs')
    def test_shrink_share(self, share_proto):
        share = fake_share.fake_share(share_proto=share_proto,
                                      size=5,
                                      host="fake_host@fake_backend#fake_pool")
        expect_share_path = self.driver.helper._generate_share_path(
            'manila_fakeid')
        mock_gns = self.mock_object(rest_helper.RestHelper,
                                    '_get_nfs_share',
                                    mock.Mock(return_value={
                                        "path": "fake_path",
                                        "clients": ["client"],
                                        "protocol": "fake_protocol"
                                    }))
        mock_gcs = self.mock_object(rest_helper.RestHelper, '_get_cifs_share',
                                    mock.Mock(return_value={
                                        "path": "fake_path",
                                        "cifsname": "fake_cifsname",
                                        "protocol": "fake_protocol",
                                        "roList": ["fake_ro"],
                                        "rwList": ["fake_rw"],
                                        "allowList": ["fake_allow"],
                                        "denyList": ["fake_deny"]
                                    }))
        mock_gf = self.mock_object(rest_helper.RestHelper, '_get_filesystem',
                                   mock.Mock(return_value={
                                       "name": "fake_name",
                                       "poolName": "fake_pool",
                                       "quotaStatus": "5GB",
                                       "usedCapacity": '1GB'
                                   }))
        mock_uss = self.mock_object(rest_helper.RestHelper,
                                    '_update_share_size')
        self.driver.shrink_share(share, 3)
        if share_proto == 'nfs':
            mock_gns.assert_called_once_with(expect_share_path)
        else:
            mock_gcs.assert_called_once_with(expect_share_path)

        mock_gf.assert_called_once_with('manila_fakeid')
        mock_uss.assert_called_once_with('manila_fakeid', '3GB')

    @ddt.data('nfs', 'cifs')
    def test_shrink_share_not_exist(self, share_proto):
        share = fake_share.fake_share(share_proto=share_proto,
                                      size=3,
                                      host="fake_host@fake_backend#fake_pool")
        expect_share_path = self.driver.helper._generate_share_path(
            'manila_fakeid')
        mock_gns = self.mock_object(rest_helper.RestHelper, '_get_nfs_share',
                                    mock.Mock(return_value=None))
        mock_gcs = self.mock_object(rest_helper.RestHelper, '_get_cifs_share',
                                    mock.Mock(return_value=None))

        self.assertRaises(exception.ShareResourceNotFound,
                          self.driver.shrink_share,
                          share,
                          1)
        if share_proto == 'nfs':
            mock_gns.assert_called_once_with(expect_share_path)
        elif share_proto == 'cifs':
            mock_gcs.assert_called_once_with(expect_share_path)

    def test_shrink_share_size_fail(self):
        share = fake_share.fake_share(share_proto='nfs',
                                      size=3,
                                      host="fake_host@fake_backend#fake_pool")
        expect_share_path = self.driver.helper._generate_share_path(
            'manila_fakeid')
        mock_gns = self.mock_object(rest_helper.RestHelper,
                                    '_get_nfs_share',
                                    mock.Mock(return_value={
                                        "path": "fake_path",
                                        "clients": ["client"],
                                        "protocol": "fake_protocol"
                                    }))
        mock_gf = self.mock_object(rest_helper.RestHelper, '_get_filesystem',
                                   mock.Mock(return_value={
                                       "name": "fake_name",
                                       "poolName": "fake_pool",
                                       "quotaStatus": "3GB",
                                       "usedCapacity": '2GB'
                                   }))
        self.assertRaises(exception.ShareShrinkingPossibleDataLoss,
                          self.driver.shrink_share,
                          share,
                          1)
        mock_gf.assert_called_once_with('manila_fakeid')
        mock_gns.assert_called_once_with(expect_share_path)

    @ddt.data('nfs', 'cifs')
    def test_ensure_share(self, share_proto):
        share = fake_share.fake_share(share_proto=share_proto,
                                      host="fake_host@fake_backend#fake_pool")
        mock_gns = self.mock_object(rest_helper.RestHelper, '_get_nfs_share',
                                    mock.Mock(return_value={
                                        "path": "fake_path",
                                        "clients": ["client"],
                                        "protocol": "fake_protocol"
                                    }))
        mock_gcs = self.mock_object(rest_helper.RestHelper, '_get_cifs_share',
                                    mock.Mock(return_value={
                                        "path": "fake_path",
                                        "cifsname": "fake_cifsname",
                                        "protocol": "fake_protocol",
                                        "roList": ["fake_ro"],
                                        "rwList": ["fake_rw"],
                                        "allowList": ["fake_allow"],
                                        "denyList": ["fake_deny"],
                                    }))
        self.driver.helper.configuration.macrosan_nas_ip = "172.0.0.1"
        locations = self.driver.ensure_share(self._context, share)
        expect_share_path = self.driver.helper._generate_share_path(
            'manila_fakeid')
        if share_proto == 'nfs':
            expect_locations = [r'172.0.0.1:/manila_fakeid/manila_fakeid']
            self.assertEqual(locations, expect_locations)
            mock_gns.assert_called_once_with(expect_share_path)
        else:
            expect_locations = [r'\\172.0.0.1\manila_fakeid']
            self.assertEqual(locations, expect_locations)
            mock_gcs.assert_called_once_with(expect_share_path)

    def test_ensure_share_proto_fail(self):
        share = fake_share.fake_share(host="fake_host@fake_backend#fake_pool")
        self.mock_object(rest_helper.RestHelper, '_get_nfs_share',
                         mock.Mock(return_value={
                             "path": "fake_path",
                             "clients": ["client"],
                             "protocol": "fake_protocol"
                         }))
        self.assertRaises(exception.MacrosanBackendExeption,
                          self.driver.ensure_share,
                          self._context,
                          share)

    def test_ensure_share_not_exist(self):
        share = fake_share.fake_share(share_proto='nfs',
                                      host="fake_host@fake_backend#fake_pool")
        mock_gns = self.mock_object(rest_helper.RestHelper, '_get_nfs_share',
                                    mock.Mock(return_value=None))
        self.assertRaises(exception.ShareResourceNotFound,
                          self.driver.ensure_share,
                          self._context,
                          share)
        expect_share_path = self.driver.helper._generate_share_path(
            'manila_fakeid')
        mock_gns.assert_called_once_with(expect_share_path)

    @ddt.data('nfs', 'cifs')
    def test_allow_access_success(self, share_proto):
        share = fake_share.fake_share(share_proto=share_proto,
                                      host="fake_host@fake_backend#fake_pool")
        if share_proto == 'nfs':
            access = {
                'access_type': 'ip',
                'access_to': '0.0.0.0/0',
                'access_level': 'rw',
            }
        else:
            access = {
                'access_type': 'user',
                'access_to': 'fake_user',
                'access_level': 'rw',
            }
        mock_gns = self.mock_object(rest_helper.RestHelper,
                                    '_get_nfs_share',
                                    mock.Mock(return_value={
                                        "path": "fake_path",
                                        "clients": ["client"],
                                        "protocol": "fake_protocol"
                                    }))
        mock_gafns = self.mock_object(rest_helper.RestHelper,
                                      '_get_access_from_nfs_share',
                                      mock.Mock(return_value=None))
        mock_anar = self.mock_object(rest_helper.RestHelper,
                                     '_allow_nfs_access_rest')
        mock_gcs = self.mock_object(rest_helper.RestHelper,
                                    '_get_cifs_share',
                                    mock.Mock(return_value={
                                        "path": "fake_path",
                                        "cifsname": "fake_cifsname",
                                        "protocol": "fake_protocol",
                                        "roList": ["fake_ro"],
                                        "rwList": ["fake_rw"],
                                        "allowList": ["fake_allow"],
                                        "denyList": ["fake_deny"],
                                    }))
        mock_gafcs = self.mock_object(rest_helper.RestHelper,
                                      '_get_access_from_cifs_share',
                                      mock.Mock(return_value=None))
        mock_acar = self.mock_object(rest_helper.RestHelper,
                                     '_allow_cifs_access_rest')
        self.driver.helper._allow_access(share, access)
        expect_share_path = self.driver.helper._generate_share_path(
            'manila_fakeid')
        if access['access_to'] == '0.0.0.0/0':
            access['access_to'] = '*'
        if share_proto == 'nfs':
            mock_gns.assert_called_once_with(expect_share_path)
            mock_gafns.assert_called_once_with(expect_share_path,
                                               access['access_to'])
            mock_anar.assert_called_once_with(expect_share_path,
                                              access['access_to'],
                                              access['access_level'])
        else:
            mock_gcs.assert_called_once_with(expect_share_path)
            mock_gafcs.assert_called_once_with(expect_share_path,
                                               access['access_to'])
            mock_acar.assert_called_once_with(expect_share_path,
                                              access['access_to'],
                                              access['access_level'])

    def test_allow_access_nfs_change(self):
        share = fake_share.fake_share(share_proto='nfs',
                                      host="fake_host@fake_backend#fake_pool")
        access = {
            'access_type': 'ip',
            'access_to': '172.0.0.1',
            'access_level': 'rw',
        }
        mock_gns = self.mock_object(rest_helper.RestHelper, '_get_nfs_share',
                                    mock.Mock(return_value={
                                        "path": "/manila_fakeid",
                                        "clients": ["client"],
                                        "protocol": "fake_protocol"
                                    }))
        mock_gafns = self.mock_object(rest_helper.RestHelper,
                                      '_get_access_from_nfs_share',
                                      mock.Mock(return_value={
                                          "path": "/manila_fakeid",
                                          "clientName": "fake_client_name",
                                          "accessRight": "ro",
                                      }))
        mock_cnar = self.mock_object(rest_helper.RestHelper,
                                     '_change_nfs_access_rest')
        self.driver.helper._allow_access(share, access)
        expect_share_path = self.driver.helper._generate_share_path(
            'manila_fakeid')
        mock_gns.assert_called_once_with(expect_share_path)
        mock_gafns.assert_called_once_with(expect_share_path,
                                           access['access_to'])
        mock_cnar.assert_called_once_with(expect_share_path,
                                          access['access_to'],
                                          access['access_level'])

    def test_allow_access_cifs_change(self):
        share = fake_share.fake_share(share_proto='cifs',
                                      host="fake_host@fake_backend#fake_pool")
        access = {
            'access_type': 'user',
            'access_to': 'fake_user',
            'access_level': 'rw',
        }
        mock_gcs = self.mock_object(rest_helper.RestHelper,
                                    '_get_cifs_share',
                                    mock.Mock(return_value={
                                        "path": "fake_path",
                                        "cifsname": "fake_cifsname",
                                        "protocol": "fake_protocol",
                                        "roList": ["fake_ro"],
                                        "rwList": ["fake_rw"],
                                        "allowList": ["fake_allow"],
                                        "denyList": ["fake_deny"],
                                    }))
        mock_gafcs = self.mock_object(rest_helper.RestHelper,
                                      '_get_access_from_cifs_share',
                                      mock.Mock(return_value={
                                          "path": "fake_path",
                                          "ugName": "fake_user",
                                          "ugType": "0",
                                          "accessRight": "ro",
                                      }))
        mock_ccar = self.mock_object(rest_helper.RestHelper,
                                     '_change_cifs_access_rest')
        self.driver.helper._allow_access(share, access)
        expect_share_path = self.driver.helper._generate_share_path(
            'manila_fakeid')
        mock_gcs.assert_called_once_with(expect_share_path)
        mock_gafcs.assert_called_once_with(expect_share_path,
                                           access['access_to'])
        mock_ccar.assert_called_once_with(expect_share_path,
                                          access['access_to'],
                                          access['access_level'],
                                          '0')

    @ddt.data(
        {
            'access_type': 'user',
            'access_to': 'user_name',
            'access_level': 'rw',
        },
        {
            'access_type': 'user',
            'access_to': 'group_name',
            'access_level': 'rw',
        },
        {
            'access_type': 'user',
            'access_to': '/domain_user',
            'access_level': 'rw',
        },
        {
            'access_type': 'user',
            'access_to': '/domain_group',
            'access_level': 'rw',
        },
    )
    def test_allow_access_cifs(self, access):
        share = fake_share.fake_share(share_proto='cifs',
                                      host="fake_host@fake_backend#fake_pool")
        mock_gcs = self.mock_object(rest_helper.RestHelper,
                                    '_get_cifs_share',
                                    mock.Mock(return_value={
                                        "path": "fake_path",
                                        "cifsname": "fake_cifsname",
                                        "protocol": "fake_protocol",
                                        "roList": ["fake_ro"],
                                        "rwList": ["fake_rw"],
                                        "allowList": ["fake_allow"],
                                        "denyList": ["fake_deny"],
                                    }))
        mock_gafcs = self.mock_object(rest_helper.RestHelper,
                                      '_get_access_from_cifs_share',
                                      mock.Mock(return_value=None))
        mock_acar = self.mock_object(rest_helper.RestHelper,
                                     '_allow_cifs_access_rest')

        self.driver.helper._allow_access(share, access)
        expect_share_path = self.driver.helper._generate_share_path(
            'manila_fakeid')
        mock_gcs.assert_called_once_with(expect_share_path)
        mock_gafcs.assert_called_once_with(expect_share_path,
                                           access['access_to'])
        mock_acar.assert_called_once_with(expect_share_path,
                                          access['access_to'],
                                          access['access_level'])

    @ddt.data('nfs', 'cifs')
    def test_allow_access_share_not_exist(self, share_proto):
        share = fake_share.fake_share(share_proto=share_proto,
                                      host="fake_host@fake_backend#fake_pool")
        access = {}
        if share_proto == 'nfs':
            access = {
                'access_type': 'ip',
                'access_to': '172.0.0.1',
                'access_level': 'rw',

            }
        else:
            access = {
                'access_type': 'user',
                'access_to': 'fake_user',
                'access_level': 'rw',
            }
        mock_gns = self.mock_object(rest_helper.RestHelper, '_get_nfs_share',
                                    mock.Mock(return_value=None))
        mock_gcs = self.mock_object(rest_helper.RestHelper, '_get_cifs_share',
                                    mock.Mock(return_value=None))
        self.assertRaises(exception.ShareResourceNotFound,
                          self.driver.helper._allow_access,
                          share,
                          access)
        expect_share_path = self.driver.helper._generate_share_path(
            'manila_fakeid')
        if share_proto == 'nfs':
            mock_gns.assert_called_once_with(expect_share_path)
        else:
            mock_gcs.assert_called_once_with(expect_share_path)

    def test_allow_access_proto_fail(self):
        share = fake_share.fake_share(host="fake_host@fake_backend#fake_pool")
        access = {
            'access_type': 'user',
            'access_to': 'fake_user',
            'access_level': 'rw',
        }
        self.assertRaises(exception.MacrosanBackendExeption,
                          self.driver.helper._allow_access,
                          share,
                          access)

    def test_allow_access_nfs_user_fail(self):
        share = fake_share.fake_share(share_proto='nfs',
                                      host="fake_host@fake_backend#fake_pool")
        access = {
            'access_type': 'user',
            'access_to': 'fake_user',
            'access_level': 'rw',
        }
        self.assertRaises(exception.InvalidShareAccess,
                          self.driver.helper._allow_access,
                          share,
                          access)

    def test_allow_access_cifs_ip_fail(self):
        share = fake_share.fake_share(share_proto='cifs',
                                      host="fake_host@fake_backend#fake_pool")
        access = {
            'access_type': 'ip',
            'access_to': '172.0.0.1',
            'access_level': 'rw',
        }
        self.assertRaises(exception.InvalidShareAccess,
                          self.driver.helper._allow_access,
                          share,
                          access)

    def test_allow_access_nfs_level_fail(self):
        share = fake_share.fake_share(share_proto='nfs',
                                      host="fake_host@fake_backend#fake_pool")
        access = {
            'access_type': 'ip',
            'access_to': '172.0.0.1',
            'access_level': 'r',
        }
        self.assertRaises(exception.InvalidShareAccess,
                          self.driver.helper._allow_access,
                          share,
                          access)

    @ddt.data('nfs', 'cifs')
    def test_deny_access(self, share_proto):
        share = fake_share.fake_share(share_proto=share_proto,
                                      host="fake_host@fake_backend#fake_pool")
        if share_proto == 'nfs':
            access = {
                'access_type': 'ip',
                'access_to': '0.0.0.0/0',
                'access_level': 'rw',
            }
        else:
            access = {
                'access_type': 'user',
                'access_to': 'fake_user',
                'access_level': 'rw',
            }
        mock_gafns = self.mock_object(rest_helper.RestHelper,
                                      '_get_access_from_nfs_share',
                                      mock.Mock(return_value={
                                          "path": "fake_path",
                                          "clientName": "fake_client_name",
                                          "accessRight": "rw",
                                      }))
        mock_dnar = self.mock_object(rest_helper.RestHelper,
                                     '_delete_nfs_access_rest')
        mock_gafcs = self.mock_object(rest_helper.RestHelper,
                                      '_get_access_from_cifs_share',
                                      mock.Mock(return_value={
                                          "path": "fake_path",
                                          "ugName": "fake_user",
                                          "ugType": "0",
                                          "accessRight": "rw",
                                      }))
        mock_dcar = self.mock_object(rest_helper.RestHelper,
                                     '_delete_cifs_access_rest')
        expect_share_path = self.driver.helper._generate_share_path(
            'manila_fakeid')
        self.driver.helper._deny_access(share, access)
        if access['access_to'] == '0.0.0.0/0':
            access['access_to'] = '*'
        if share_proto == 'nfs':
            mock_gafns.assert_called_once_with(expect_share_path,
                                               access['access_to'])
            mock_dnar.assert_called_once_with(expect_share_path,
                                              access['access_to'])
        else:
            mock_gafcs.assert_called_once_with(expect_share_path,
                                               access['access_to'])
            mock_dcar.assert_called_once_with(expect_share_path,
                                              "fake_user", "0")

    def test_deny_access_nfs_type_fail(self):
        share = fake_share.fake_share(share_proto='nfs',
                                      host="fake_host@fake_backend#fake_pool")
        access = {
            'access_type': 'fake_type',
            'access_to': '172.0.0.1',
            'access_level': 'rw',
        }
        result = self.driver.helper._deny_access(share, access)
        self.assertIsNone(result)

    def test_deny_access_nfs_share_not_exist(self):
        share = fake_share.fake_share(share_proto='nfs',
                                      host="fake_host@fake_backend#fake_pool")
        access = {
            'access_type': 'ip',
            'access_to': '172.0.0.1',
            'access_level': 'rw',
        }
        mock_gafns = self.mock_object(rest_helper.RestHelper,
                                      '_get_access_from_nfs_share',
                                      mock.Mock(return_value=None))
        result = self.driver.helper._deny_access(share, access)
        self.assertIsNone(result)
        expect_share_path = self.driver.helper._generate_share_path(
            'manila_fakeid')
        mock_gafns.assert_called_once_with(expect_share_path,
                                           access['access_to'])

    def test_deny_access_cifs_type_fail(self):
        share = fake_share.fake_share(share_proto='cifs',
                                      host="fake_host@fake_backend#fake_pool")
        access = {
            'access_type': 'fake_type',
            'access_to': 'fake_user',
            'access_level': 'rw',
        }
        result = self.driver.helper._deny_access(share, access)
        self.assertIsNone(result)

    def test_deny_access_cifs_share_not_exist(self):
        share = fake_share.fake_share(share_proto='cifs',
                                      host="fake_host@fake_backend#fake_pool")
        access = {
            'access_type': 'user',
            'access_to': 'fake_user',
            'access_level': 'rw',
        }
        mock_gafcs = self.mock_object(rest_helper.RestHelper,
                                      '_get_access_from_cifs_share',
                                      mock.Mock(return_value=None))
        result = self.driver.helper._deny_access(share, access)
        self.assertIsNone(result)
        expect_share_path = self.driver.helper._generate_share_path(
            'manila_fakeid')
        mock_gafcs.assert_called_once_with(expect_share_path,
                                           access['access_to'])

    def test_update_access_add_delete(self):
        share = fake_share.fake_share(share_proto='nfs',
                                      host="fake_host@fake_backend#fake_pool")
        add_rules = [{'access_type': 'ip',
                      'access_to': '172.0.2.1',
                      'access_level': 'rw', }]
        delete_rules = [{'access_type': 'ip',
                         'access_to': '172.0.2.2',
                         'access_level': 'rw', }]
        self.mock_object(macrosan_helper.MacrosanHelper,
                         '_allow_access')
        self.mock_object(macrosan_helper.MacrosanHelper,
                         '_deny_access')
        self.driver.update_access(self._context, share,
                                  None, add_rules, delete_rules)

    @ddt.data('nfs', 'cifs')
    def test_update_access_nfs(self, proto):
        share = fake_share.fake_share(share_proto=proto,
                                      host="fake_host@fake_backend#fake_pool")
        if proto == 'nfs':
            access_rules = [{'access_type': 'ip',
                             'access_to': '172.0.3.1',
                             'access_level': 'rw', },
                            {'access_type': 'ip',
                             'access_to': '172.0.3.2',
                             'access_level': 'rw', }]
        else:
            access_rules = [{'access_type': 'user',
                             'access_to': 'user_l',
                             'access_level': 'rw', },
                            {'access_type': 'user',
                             'access_to': 'user_a',
                             'access_level': 'rw', }]
        mock_ca = self.mock_object(macrosan_helper.MacrosanHelper,
                                   '_clear_access')
        self.mock_object(macrosan_helper.MacrosanHelper,
                         '_allow_access')
        self.driver.update_access(self._context, share,
                                  access_rules, {}, {})
        mock_ca.assert_called_once_with(share, None)

    def test_update_access_fail(self):
        share = fake_share.fake_share(share_proto='nfs',
                                      host="fake_host@fake_backend#fake_pool")
        access_rules = [{'access_id': 'fakeid',
                         'access_type': 'ip',
                         'access_to': '172.0.3.1',
                         'access_level': 'rw', }]
        mock_ca = self.mock_object(macrosan_helper.MacrosanHelper,
                                   '_clear_access')
        self.mock_object(macrosan_helper.MacrosanHelper,
                         '_allow_access',
                         mock.Mock(side_effect=exception.InvalidShareAccess(
                             reason='fake_exception')))
        result = self.driver.update_access(self._context, share,
                                           access_rules, None, None)
        expect = {
            'fakeid': {
                'state': 'error',
            }
        }
        self.assertEqual(result, expect)
        mock_ca.assert_called_once_with(share, None)

    def test_update_access_add_fail(self):
        share = fake_share.fake_share(share_proto='nfs',
                                      host="fake_host@fake_backend#fake_pool")
        add_rules = [{'access_id': 'fakeid',
                      'access_type': 'ip',
                      'access_to': '172.0.2.1',
                      'access_level': 'rw', }]
        delete_rules = []
        self.mock_object(macrosan_helper.MacrosanHelper,
                         '_allow_access',
                         mock.Mock(side_effect=exception.InvalidShareAccess(
                             reason='fake_exception')))
        self.mock_object(macrosan_helper.MacrosanHelper,
                         '_deny_access')
        result = self.driver.update_access(self._context, share,
                                           None, add_rules, delete_rules)
        expect = {
            'fakeid': {
                'state': 'error'
            }
        }
        self.assertEqual(result, expect)

    @ddt.data('nfs', 'cifs')
    def test__clear_access(self, share_proto):
        share = fake_share.fake_share(share_proto=share_proto,
                                      host="fake_host@fake_backend#fake_pool")
        fake_nfs_share_backend = [
            {
                'share_path': 'fake_path',
                'access_to': '172.0.0.1',
                'access_level': 'rw'
            },
            {
                'share_path': 'default_path',
                'access_to': '172.0.0.2',
                'access_level': 'rw'
            }]
        fake_cifs_share_backend = [
            {
                'share_path': 'fake_path',
                'access_to': 'user_name',
                'ugType': '0',
                'access_level': 'rw'
            },
            {
                'share_path': 'default_path',
                'access_to': 'manilanobody',
                'ugType': '0',
                'access_level': 'rw'
            }]
        mock_ganar = self.mock_object(
            rest_helper.RestHelper,
            '_get_all_nfs_access_rest',
            mock.Mock(return_value=fake_nfs_share_backend))
        mock_gacar = self.mock_object(
            rest_helper.RestHelper, '_get_all_cifs_access_rest',
            mock.Mock(return_value=fake_cifs_share_backend))
        self.mock_object(rest_helper.RestHelper,
                         '_delete_nfs_access_rest')
        self.mock_object(rest_helper.RestHelper,
                         '_delete_cifs_access_rest')
        self.driver.helper._clear_access(share)
        expect_share_path = self.driver.helper._generate_share_path(
            'manila_fakeid')
        if share_proto == 'nfs':
            mock_ganar.assert_called_once_with(expect_share_path)
        else:
            mock_gacar.assert_called_once_with(expect_share_path)

    @ddt.data('nfs', 'cifs')
    def test__clear_access_no_access_list(self, share_proto):
        share = fake_share.fake_share(share_proto=share_proto,
                                      host="fake_host@fake_backend#fake_pool")
        mock_ganar = self.mock_object(
            rest_helper.RestHelper,
            '_get_all_nfs_access_rest',
            mock.Mock(return_value=[]))
        mock_gacar = self.mock_object(
            rest_helper.RestHelper, '_get_all_cifs_access_rest',
            mock.Mock(return_value=[]))
        self.driver.helper._clear_access(share)
        expect_share_path = self.driver.helper._generate_share_path(
            'manila_fakeid')
        if share_proto == 'nfs':
            mock_ganar.assert_called_once_with(expect_share_path)
        else:
            mock_gacar.assert_called_once_with(expect_share_path)

    @ddt.data(constants.USER_NOT_EXIST,
              constants.USER_EXIST,
              constants.USER_FORMAT_ERROR)
    def test__ensure_user(self, query_result):
        mock_qu = self.mock_object(rest_helper.RestHelper,
                                   '_query_user',
                                   mock.Mock(return_value=query_result))
        mock_qg = self.mock_object(
            rest_helper.RestHelper,
            '_query_group',
            mock.Mock(return_value=constants.GROUP_NOT_EXIST))
        mock_alg = self.mock_object(rest_helper.RestHelper,
                                    '_add_localgroup')
        mock_alu = self.mock_object(rest_helper.RestHelper,
                                    '_add_localuser')
        result = self.driver.helper._ensure_user('fake_user',
                                                 'fake_passwd',
                                                 'fake_group')
        if query_result == constants.USER_NOT_EXIST:
            mock_qg.assert_called_once_with('fake_group')
            mock_alg.assert_called_once_with('fake_group')
            mock_alu.assert_called_once_with('fake_user',
                                             'fake_passwd',
                                             'fake_group')
            self.assertTrue(result)
        elif query_result == constants.USER_EXIST:
            self.assertTrue(result)
        else:
            self.assertFalse(result)
        mock_qu.assert_called_once_with('fake_user')

    def test__ensure_user_fail(self):
        mock_qu = self.mock_object(
            rest_helper.RestHelper,
            '_query_user',
            mock.Mock(return_value=constants.USER_NOT_EXIST))
        mock_qg = self.mock_object(
            rest_helper.RestHelper,
            '_query_group',
            mock.Mock(return_value=constants.GROUP_FORMAT_ERROR))
        self.assertRaises(exception.InvalidInput,
                          self.driver.helper._ensure_user,
                          'fake_user',
                          'fake_passwd',
                          'fake_group')
        mock_qu.assert_called_once_with('fake_user')
        mock_qg.assert_called_once_with('fake_group')

    def test__update_share_stats(self):
        self.driver.helper.pools = ['fake_pool']
        mock_gap = self.mock_object(rest_helper.RestHelper,
                                    '_get_all_pool',
                                    mock.Mock(return_value='fake_result'))
        mock_gpc = self.mock_object(macrosan_helper.MacrosanHelper,
                                    '_get_pool_capacity',
                                    mock.Mock(return_value={
                                        "totalcapacity": 10,
                                        "freecapacity": 9,
                                        "allocatedcapacity": 1,
                                    }))
        mock_uss = self.mock_object(driver.ShareDriver, '_update_share_stats')

        self.driver._update_share_stats()

        data = {}
        data['vendor_name'] = self.driver.VENDOR
        data['driver_version'] = self.driver.VERSION
        data['storage_protocol'] = self.driver.PROTOCOL
        data['share_backend_name'] = 'fake_share_backend_name'
        data['pools'] = [{
            'pool_name': 'fake_pool',
            'total_capacity_gb': 10,
            'free_capacity_gb': 9,
            'allocated_capacity_gb': 1,
            'reserved_percentage': 0,
            'reserved_snapshot_percentage': 0,
            'reserved_share_extend_percentage': 0,
            'dedupe': False,
            'compression': False,
            'qos': False,
            'thin_provisioning': False,
            'snapshot_support': False,
            'create_share_from_snapshot_support':
                False,
        }]
        mock_gap.assert_called_once()
        mock_gpc.assert_called_once_with('fake_pool', 'fake_result')
        mock_uss.assert_called_once_with(data)

    def test__update_share_stats_pool_not_exist(self):
        self.driver.helper.pools = ['fake_pool']
        self.mock_object(rest_helper.RestHelper, '_get_all_pool',
                         mock.Mock(return_value='fake_result'))
        self.mock_object(macrosan_helper.MacrosanHelper,
                         '_get_pool_capacity',
                         mock.Mock(return_value={}))
        self.assertRaises(exception.InvalidInput,
                          self.driver._update_share_stats
                          )

    def test__get_pool_capacity(self):
        self.mock_object(macrosan_helper.MacrosanHelper,
                         '_find_pool_info',
                         mock.Mock(return_value={
                             "name": "fake_pool",
                             "totalcapacity": "100.0G",
                             "allocatedcapacity": "22G",
                             "freecapacity": "78G",
                             "health": "ONLINE",
                             "rw": "off",
                         }))
        res = self.driver.helper._get_pool_capacity("fake_pool",
                                                    "fake_result")
        self.assertEqual(100, res['totalcapacity'])
        self.assertEqual(78, res['freecapacity'])
        self.assertEqual(22, res['allocatedcapacity'])

    def test__generate_share_name(self):
        share = fake_share.fake_share(host="fake_host@fake_backend#fake_pool")
        result = self.driver.helper._generate_share_name(share)
        self.assertEqual("manila_fakeid", result)

    def test__format_name(self):
        a = 'fake-1234567890-1234567890-1234567890'
        expect = 'fake_1234567890_1234567890_1234'
        result = self.driver.helper._format_name(a)
        self.assertEqual(expect, result)

    def test__generate_share_path(self):
        share_name = 'manila_fakeid'
        result = self.driver.helper._generate_share_path(share_name)

        self.assertEqual(r'/manila_fakeid/manila_fakeid', result)

    @ddt.data('nfs', 'cifs')
    def test__get_location_path(self, share_proto):
        self.driver.helper.configuration.macrosan_nas_ip = "172.0.0.1"
        result = self.driver.helper._get_location_path('fake_path',
                                                       'fake_name',
                                                       share_proto)
        if share_proto == 'nfs':
            expect = r'172.0.0.1:fake_path'
        elif share_proto == 'cifs':
            expect = r'\\172.0.0.1\fake_name'
        self.assertEqual(expect, result)

    def test__get_share_instance_pnp_pool_error(self):
        share = fake_share.fake_share(
            share_proto="nfs", host="fake_host@fake_backend")
        self.assertRaises(exception.InvalidHost,
                          self.driver.helper._get_share_instance_pnp,
                          share)

    def test__get_share_instance_pnp_proto_error(self):
        share = fake_share.fake_share(
            share_proto="CephFS", host="fake_host@fake_backend#fake_pool")
        self.assertRaises(exception.MacrosanBackendExeption,
                          self.driver.helper._get_share_instance_pnp,
                          share)

    @ddt.data('2000000000', '2000000KB', '2000MB', '20GB', '2TB')
    def test__unit_convert_toGB(self, capacity):
        convert = {'2000000000': '%.0f' % (float(2000000000) / 1024 ** 3),
                   '2000000KB': '%.0f' % (float(2000000) / 1024 ** 2),
                   '2000MB': '%.0f' % (float(2000) / 1024),
                   '20GB': '%.0f' % float(20),
                   '2TB': '%.0f' % (float(2) * 1024)}
        expect = float(convert[capacity])
        result = self.driver.helper._unit_convert_toGB(capacity)
        self.assertEqual(expect, result)

    @ddt.data('nfs', 'cifs')
    def test__get_share(self, proto):
        proto = proto.upper()
        mock_gns = self.mock_object(rest_helper.RestHelper,
                                    '_get_nfs_share',
                                    mock.Mock(return_value={
                                        "path": "/manila_fakeid",
                                        "clients": ["client"],
                                        "protocol": "NFS"
                                    }))
        mock_gcs = self.mock_object(rest_helper.RestHelper,
                                    '_get_cifs_share',
                                    mock.Mock(return_value={
                                        "path": "fake_path",
                                        "cifsname": "fake_cifsname",
                                        "protocol": "CIFS",
                                        "roList": ["fake_ro"],
                                        "rwList": ["fake_rw"],
                                        "allowList": ["fake_allow"],
                                        "denyList": ["fake_deny"],
                                    }))
        expect_nfs = {
            "path": "/manila_fakeid",
            "clients": ["client"],
            "protocol": "NFS"}
        expect_cifs = {
            "path": "fake_path",
            "cifsname": "fake_cifsname",
            "protocol": "CIFS",
            "roList": ["fake_ro"],
            "rwList": ["fake_rw"],
            "allowList": ["fake_allow"],
            "denyList": ["fake_deny"]}
        result = self.driver.helper._get_share('fake_path', proto)
        if proto == 'NFS':
            mock_gns.assert_called_once_with('fake_path')
            self.assertEqual(expect_nfs, result)
        elif proto == 'CIFS':
            mock_gcs.assert_called_once_with('fake_path')
            self.assertEqual(expect_cifs, result)

    def test__find_pool_info(self):
        pool_info = self.driver.helper._find_pool_info(
            'fake_pool',
            self.resutl_success_storage_pools)
        self.assertIsNotNone(pool_info)

    def test__find_pool_info_fail(self):
        pool_info = self.driver.helper._find_pool_info(
            'error_pool',
            self.resutl_success_storage_pools)
        expect = {}
        self.assertEqual(expect, pool_info)


@ddt.ddt
class RestHelperTestCase(test.TestCase):

    def setUp(self):
        self.mock_object(CONF, '_check_required_opts')
        super(RestHelperTestCase, self).setUp()

        def _safe_get(opt):
            return getattr(self.configuration, opt)

        self.configuration = mock.Mock(spec=configuration.Configuration)
        self.configuration.safe_get = mock.Mock(side_effect=_safe_get)
        self.configuration.macrosan_nas_http_protocol = 'https'
        self.configuration.macrosan_nas_ip = 'fake_ip'
        self.configuration.macrosan_nas_port = 'fake_port'
        self.configuration.macrosan_nas_prefix = 'nas'
        self.configuration.macrosan_nas_username = 'fake_username'
        self.configuration.macrosan_nas_password = 'fake_password'
        self.configuration.macrosan_timeout = 60
        self.resthelper = rest_helper.RestHelper(
            configuration=self.configuration)
        self.post = 'POST'
        self.get = 'GET'
        self.delete = 'DELETE'
        self.put = 'PUT'
        self.fake_message = 'fake_message'
        self.result_success = {
            'code': 0,
            'message': 'success',
            'data': 'fake_data'
        }
        self.result_success_return_0 = {
            'code': 0,
            'message': 'success',
            'data': '0'
        }
        self.result_success_return_1 = {
            'code': 0,
            'message': 'success',
            'data': '1'
        }
        self.result_failed = {
            'code': 1,
            'message': 'failed',
            'data': 'fake_data'
        }
        self.result_failed_not_exist = {
            'code': constants.CODE_SOURCE_NOT_EXIST,
            'message': 'failed',
            'data': '',
        }
        self.resutl_success_storage_pools = {
            'code': 0,
            'message': 'success',
            'data': [{
                'name': 'fake_pool',
                'size': '1000.0G',
                'allocated': '100G',
                'free': '900G',
                'health': 'ONLINE',
                'rwStatus': 'off'
            }]
        }

    @ddt.data(
        {'url': 'fake_url', 'data': {'fake_data': 'fake_value'},
         'method': 'POST'},
        {'url': 'fake_url', 'data': None,
         'method': 'GET'},
        {'url': 'fake_url', 'data': {'fake_data': 'fake_value'},
         'method': 'DELETE'},
        {'url': 'fake_url', 'data': {'fake_data': 'fake_value'},
         'method': 'PUT'},
    )
    @ddt.unpack
    def test_call(self, url, data, method):
        self.resthelper._token = 'fake_token'
        request_method = method.lower()
        fake_response = FakeResponse(200, self.result_success)
        mock_request = self.mock_object(requests, request_method,
                                        mock.Mock(return_value=fake_response))
        self.resthelper.call(url, data, method)
        expected_url = ('https://%(ip)s:%(port)s/%(rest)s/%(url)s'
                        % {'ip': 'fake_ip',
                           'port': 'fake_port',
                           'rest': 'nas',
                           'url': 'fake_url'})
        header = {'Authorization': 'fake_token'}
        mock_request.assert_called_once_with(
            expected_url, data=data, headers=header,
            timeout=self.configuration.macrosan_timeout,
            verify=False)

    def test_call_method_fail(self):
        self.resthelper._token = 'fake_token'
        self.assertRaises(exception.ShareBackendException,
                          self.resthelper.call,
                          'fake_url',
                          'fake_data',
                          'error_method')

    def test_call_token_fail(self):
        self.resthelper._token = 'fake_token'
        fake_result_fail = {
            'code': 302,
            'message': 'fake_message',
            'data': 'fake_data'
        }
        self.mock_object(self.resthelper, 'do_request',
                         mock.Mock(return_value=fake_result_fail))
        self.assertRaises(exception.MacrosanBackendExeption,
                          self.resthelper.call,
                          'fake_url',
                          'fake_data',
                          self.post)

    def test_call_token_none(self):
        self.resthelper._token = None
        self.mock_object(self.resthelper, 'do_request',
                         mock.Mock(return_value=self.result_success))
        mock_l = self.mock_object(self.resthelper, 'login',
                                  mock.Mock(return_value='fake_token'))
        self.resthelper.call('fake_url', 'fake_data', self.post)
        mock_l.assert_called_once()

    def test_call_token_expired(self):
        self.resthelper._token = 'fake_token'
        fake_result = {
            'code': 301,
            'message': 'token expired',
            'data': 'fake_data'
        }
        self.mock_object(
            self.resthelper, 'do_request',
            mock.Mock(side_effect=[fake_result, self.result_success]))
        mock_l = self.mock_object(self.resthelper, 'login',
                                  mock.Mock(return_value='fake_token'))
        self.resthelper.call('fake_url', 'fake_data', self.post)
        mock_l.assert_called_once()

    def test_call_fail(self):
        self.resthelper._token = 'fake_token'
        fake_response = FakeResponse(302, self.result_success)
        self.mock_object(requests, 'post',
                         mock.Mock(return_value=fake_response))
        self.assertRaises(exception.NetworkException,
                          self.resthelper.call,
                          'fake_url',
                          'fake_data',
                          self.post)

    def test_login(self):
        fake_result = {
            'code': 0,
            'message': 'Login success',
            'data': 'fake_token'
        }
        mock_rd = self.mock_object(self.resthelper, 'do_request',
                                   mock.Mock(return_value=fake_result))
        self.resthelper.login()
        login_data = {'userName': self.configuration.macrosan_nas_username,
                      'userPasswd': self.configuration.macrosan_nas_password}
        mock_rd.assert_called_once_with('rest/token', login_data,
                                        self.post)
        self.assertEqual('fake_token', self.resthelper._token)

    def test_login_fail(self):
        mock_rd = self.mock_object(self.resthelper, 'do_request',
                                   mock.Mock(return_value=self.result_failed))

        self.assertRaises(exception.ShareBackendException,
                          self.resthelper.login)
        login_data = {'userName': self.configuration.macrosan_nas_username,
                      'userPasswd': self.configuration.macrosan_nas_password}
        mock_rd.assert_called_once_with('rest/token', login_data,
                                        self.post)

    def test__assert_result_code(self):
        self.resthelper._assert_result_code(self.result_success,
                                            self.fake_message)

    def test__assert_result_code_fail(self):
        self.assertRaises(exception.ShareBackendException,
                          self.resthelper._assert_result_code,
                          self.result_failed,
                          self.fake_message)

    def test__assert_result_data(self):
        self.resthelper._assert_result_data(self.result_success,
                                            self.fake_message)

    def test__assert_result_data_fail(self):
        fake_result = {
            'code': 0,
            'message': 'fake_message'
        }
        self.assertRaises(exception.ShareBackendException,
                          self.resthelper._assert_result_data,
                          fake_result,
                          self.fake_message)

    def test__create_nfs_share(self):
        mock_call = self.mock_object(self.resthelper,
                                     'call')
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        self.resthelper._create_nfs_share('fake_path')
        url = 'rest/nfsShare'
        data = {
            'path': 'fake_path',
            'authority': 'ro',
            'accessClient': '172.0.0.2',
        }
        mock_call.assert_called_once_with(url, data, self.post)

    def test__get_nfs_share(self):
        fake_result = {
            'code': 0,
            'message': 'success',
            'data': {
                "path": "fake_path",
                "clients": ["client"],
                "protocol": "fake_protocol"
            }
        }
        mock_call = self.mock_object(self.resthelper,
                                     'call',
                                     mock.Mock(return_value=fake_result))
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        result = self.resthelper._get_nfs_share('fake_path')
        expect = {
            "path": "fake_path",
            "clients": ["client"],
            "protocol": "fake_protocol"
        }
        self.assertEqual(expect, result)
        url = 'rest/nfsShare?path=fake_path'
        mock_call.assert_called_once_with(url, None, self.get)

    def test__delete_nfs_share(self):
        mock_call = self.mock_object(self.resthelper,
                                     'call')
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        self.resthelper._delete_nfs_share('fake_path')
        url = 'rest/nfsShare?path=fake_path'
        mock_call.assert_called_once_with(url, None, self.delete)

    def test__create_cifs_share(self):
        mock_call = self.mock_object(self.resthelper,
                                     'call')
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        self.resthelper._create_cifs_share('fake_name',
                                           'fake_path',
                                           ['fake_user'],
                                           ['0'])
        url = 'rest/cifsShare'
        data = {
            'path': 'fake_path',
            'cifsName': 'fake_name',
            'cifsDescription': '',
            'RoList': [],
            'RoListType': [],
            'RwList': ['fake_user'],
            'RwListType': ['0'],
            'allowList': [],
            'denyList': [],
        }
        mock_call.assert_called_once_with(url, data, self.post)

    def test__get_cifs_share(self):
        fake_result = {
            'code': 0,
            'message': 'success',
            'data': {
                "path": "fake_path",
                "cifsname": "fake_cifsname",
                "protocol": "fake_protocol",
                "roList": ["fake_ro"],
                "rwList": ["fake_rw"],
                "allowList": ["fake_allow"],
                "denyList": ["fake_deny"]
            }
        }
        mock_call = self.mock_object(self.resthelper,
                                     'call',
                                     mock.Mock(return_value=fake_result))
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        result = self.resthelper._get_cifs_share('fake_path')
        expect = {
            "path": "fake_path",
            "cifsname": "fake_cifsname",
            "protocol": "fake_protocol",
            "roList": ["fake_ro"],
            "rwList": ["fake_rw"],
            "allowList": ["fake_allow"],
            "denyList": ["fake_deny"]
        }
        self.assertEqual(expect, result)
        url = 'rest/cifsShare?path=fake_path'
        mock_call.assert_called_once_with(url, None, self.get)

    def test__delete_cifs_share(self):
        mock_call = self.mock_object(self.resthelper,
                                     'call')
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        self.resthelper._delete_cifs_share('fake_name', 'fake_path')
        url = 'rest/cifsShare?path=fake_path&cifsName=fake_name'
        mock_call.assert_called_once_with(url, None, self.delete)

    def test__update_share_size(self):
        mock_call = self.mock_object(self.resthelper,
                                     'call')
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        self.resthelper._update_share_size('fake_filesystem', '2GB')
        url = 'rest/filesystem/fake_filesystem'
        data = {
            'capacity': '2GB',
        }
        mock_call.assert_called_once_with(url, data, self.put)

    def test___create_filesystem(self):
        mock_call = self.mock_object(self.resthelper,
                                     'call')
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        self.resthelper._create_filesystem('fake_filesystem',
                                           'fake_pool',
                                           '1GB')
        url = 'rest/filesystem'
        data = {
            'fsName': 'fake_filesystem',
            'poolName': 'fake_pool',
            'createType': '0',
            'fileSystemQuota': '1GB',
            'fileSystemReserve': '1GB',
            'wormStatus': 0,
            'defaultTimeStatus': 0,
            'defaultTimeNum': 0,
            'defaultTimeUnit': 'year',
            'isAutoLock': 0,
            'isAutoDelete': 0,
            'lockTime': 0
        }
        mock_call.assert_called_once_with(url, data, self.post)

    def test__delete_filesystem(self):
        mock_call = self.mock_object(self.resthelper,
                                     'call')
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        self.resthelper._delete_filesystem('fake_filesystem')
        url = 'rest/filesystem/fake_filesystem'
        mock_call.assert_called_once_with(url, None, self.delete)

    def test__get_filesystem(self):
        fake_result = {
            'code': 0,
            'message': 'success',
            'data': {
                'name': 'fake_filesystem',
                'poolName': 'fake_pool',
            }
        }
        mock_call = self.mock_object(self.resthelper,
                                     'call',
                                     mock.Mock(return_value=fake_result))
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        result = self.resthelper._get_filesystem('fake_filesystem')
        expect = {
            'name': 'fake_filesystem',
            'poolName': 'fake_pool',
        }
        self.assertEqual(expect, result)
        url = 'rest/filesystem/fake_filesystem'
        mock_call.assert_called_once_with(url, None, self.get)

    def test__create_filesystem_dir(self):
        mock_call = self.mock_object(self.resthelper,
                                     'call')
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        self.resthelper._create_filesystem_dir('/fake_path/fake_dir')
        url = 'rest/fileDir'
        data = {
            'path': '/fake_path',
            'dirName': 'fake_dir',
        }
        mock_call.assert_called_once_with(url, data, self.post)

    def test__delete_filesystem_dir(self):
        mock_call = self.mock_object(self.resthelper,
                                     'call')
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        self.resthelper._delete_filesystem_dir('/fake_path/fake_dir')
        url = 'rest/fileDir?path=/fake_path&dirName=fake_dir'
        mock_call.assert_called_once_with(url, None, self.delete)

    @ddt.data('nfs', 'cifs')
    def test__allow_access_rest(self, share_proto):
        share_proto = share_proto.upper()
        mock_anar = self.mock_object(self.resthelper,
                                     '_allow_nfs_access_rest')
        mock_acar = self.mock_object(self.resthelper,
                                     '_allow_cifs_access_rest')
        self.resthelper._allow_access_rest('fake_path', 'fake_access',
                                           'rw', share_proto)
        if share_proto == 'NFS':
            mock_anar.assert_called_once_with('fake_path',
                                              'fake_access',
                                              'rw')
        elif share_proto == 'CIFS':
            mock_acar.assert_called_once_with('fake_path',
                                              'fake_access',
                                              'rw')

    def test__allow_access_rest_proto_error(self):
        self.assertRaises(exception.InvalidInput,
                          self.resthelper._allow_access_rest,
                          'fake_path',
                          'fake_access',
                          'rw',
                          'error_proto')

    def test__allow_nfs_access_rest(self):
        mock_call = self.mock_object(
            self.resthelper,
            'call',
            mock.Mock(return_value=self.result_success))
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        self.resthelper._allow_nfs_access_rest('fake_path', '172.0.0.1', 'rw')
        url = 'rest/nfsShareClient'
        data = {
            'path': 'fake_path',
            'client': '172.0.0.1',
            'authority': 'rw',
        }
        mock_call.assert_called_once_with(url, data, self.post)

    @ddt.data(
        {'access_to': 'fake_user',
         'group': False},
        {'access_to': 'fake_group',
         'group': True},
        {'access_to': '/fake_user',
         'group': False},
        {'access_to': '/fake_group',
         'group': True}
    )
    @ddt.unpack
    def test__allow_cifs_access_rest(self, access_to, group):
        ug_type_list = {
            'localUser': '0',
            'localGroup': '1',
            'adUser': '2',
            'adGroup': '3',
        }
        if not group:
            mock_call = self.mock_object(
                self.resthelper,
                'call',
                mock.Mock(return_value=self.result_success))
        else:
            mock_call = self.mock_object(
                self.resthelper,
                'call',
                mock.Mock(side_effect=[self.result_failed_not_exist,
                                       self.result_success]))
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        self.resthelper._allow_cifs_access_rest('fake_path',
                                                access_to,
                                                'rw')
        url = 'rest/cifsShareClient'
        actual_type = ug_type_list["localUser"]
        if '/' not in access_to:
            if not group:
                actual_type = ug_type_list["localUser"]
            access_to = access_to
        else:
            if not group:
                actual_type = ug_type_list["adUser"]
            access_to = access_to[access_to.index('/') + 1:]
        data = {
            'path': 'fake_path',
            'right': 'rw',
            'ugName': access_to,
            'ugType': actual_type,
        }
        if not group:
            mock_call.assert_called_once_with(url, data, self.post)
        else:
            mock_call.assert_called()

    def test__allow_cifs_access_rest_fail(self):
        mock_call = self.mock_object(
            self.resthelper,
            'call',
            mock.Mock(side_effect=[self.result_failed_not_exist,
                                   self.result_failed_not_exist]))
        self.assertRaises(exception.InvalidShare,
                          self.resthelper._allow_cifs_access_rest,
                          'fake_path',
                          'fake_user',
                          'rw')
        mock_call.assert_called()

    def test__get_access_from_nfs_share(self):
        fake_result = {
            'code': 0,
            'message': 'success',
            'data': {
                "path": "fake_path",
                "clientName": "fake_client",
                "accessRight": "rw",
            }
        }
        mock_call = self.mock_object(self.resthelper,
                                     'call',
                                     mock.Mock(return_value=fake_result))
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        result = self.resthelper._get_access_from_nfs_share('fake_path',
                                                            'fake_client')
        expect = {
            "path": "fake_path",
            "clientName": "fake_client",
            "accessRight": "rw",
        }
        self.assertEqual(expect, result)
        url = 'rest/nfsShareClient?path=fake_path&client=fake_client'
        mock_call.assert_called_once_with(url, None, self.get)

    @ddt.data({'access_to': 'fake_user',
               'ug_type': '0',
               'code': 0,
               'group': False},
              {'access_to': 'fake_user',
               'ug_type': None,
               'code': 0,
               'group': False},
              {'access_to': 'fake_group',
               'ug_type': None,
               'code': 0,
               'group': True},
              {'access_to': 'fake_user',
               'ug_type': None,
               'code': 4,
               'group': False},
              {'access_to': '/fake_user',
               'ug_type': None,
               'code': 0,
               'group': False},
              {'access_to': '/fake_group',
               'ug_type': None,
               'code': 0,
               'group': True},
              {'access_to': '/fake_user',
               'ug_type': None,
               'code': 4,
               'group': False})
    @ddt.unpack
    def test__get_access_from_cifs_share(self,
                                         access_to, ug_type, code, group):
        fake_result_failed = {
            'code': code,
            'message': 'failed',
            'data': {}
        }
        fake_result = {
            'code': code,
            'message': 'success',
            'data': {
                'path': 'fake_path',
                'ugName': 'fake_user',
                'ugType': '0',
                'accessRight': 'rw'
            }
        }
        fake_result_group = {
            'code': code,
            'message': 'success',
            'data': {
                'path': 'fake_path',
                'ugName': 'fake_group',
                'ugType': '1',
                'accessRight': 'rw'
            }
        }
        if code == 4:
            fake_result = fake_result_failed
        ug_type_list = {
            'localUser': '0',
            'localGroup': '1',
            'adUser': '2',
            'adGroup': '3',
        }
        expect = {
            'path': 'fake_path',
            'ugName': 'fake_user',
            'ugType': '0',
            'accessRight': 'rw'
        }
        expect_group = {
            'path': 'fake_path',
            'ugName': 'fake_group',
            'ugType': '1',
            'accessRight': 'rw'
        }
        if '/' in access_to:
            expect['ugType'] = '2'
            expect_group['ugType'] = '3'
            fake_result['data']['ugType'] = '2'
            fake_result_group['data']['ugType'] = '3'
        if ug_type is not None:
            mock_call = self.mock_object(self.resthelper,
                                         'call',
                                         mock.Mock(return_value=fake_result))
        else:
            if not group:
                mock_call = self.mock_object(
                    self.resthelper,
                    'call',
                    mock.Mock(return_value=fake_result))
            else:
                mock_call = self.mock_object(
                    self.resthelper,
                    'call',
                    mock.Mock(side_effect=[fake_result_failed,
                                           fake_result_group]))

        self.mock_object(self.resthelper,
                         '_assert_result_code')
        result = self.resthelper._get_access_from_cifs_share('fake_path',
                                                             access_to,
                                                             ug_type)
        if ug_type:
            self.assertEqual(expect, result)
            url = f'rest/cifsShareClient?path=fake_path&' \
                  f'ugName={access_to}&ugType={ug_type}'
            mock_call.assert_called_once_with(url, None, self.get)
        else:
            if '/' not in access_to:
                if not group:
                    actual_type = ug_type_list["localUser"]
                actual_access = access_to
            else:
                if not group:
                    actual_type = ug_type_list["adUser"]
                actual_access = access_to[access_to.index('/') + 1:]
            if code == 4:
                self.assertIsNone(result)
            else:
                if not group:
                    self.assertEqual(expect, result)
                    url = f'rest/cifsShareClient?path=fake_path&' \
                          f'ugName={actual_access}&' \
                          f'ugType={actual_type}'
                    mock_call.assert_called_once_with(url, None, self.get)
                else:
                    self.assertEqual(expect_group, result)
                    mock_call.assert_called()

    def test__get_all_nfs_access_rest(self):
        fake_result = {
            'code': 0,
            'message': 'success',
            'data': [
                {
                    'path': 'fake_path',
                    'clientName': '172.0.0.1',
                    'accessRight': 'rw'
                },
                {
                    'path': 'default_path',
                    'clientName': '172.0.0.2',
                    'accessRight': 'rw'
                }]
        }
        mock_call = self.mock_object(self.resthelper,
                                     'call',
                                     mock.Mock(return_value=fake_result))
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        result = self.resthelper._get_all_nfs_access_rest(
            '/manila_fakeid/manila_fakeid')
        expect = [
            {
                'share_path': 'fake_path',
                'access_to': '172.0.0.1',
                'access_level': 'rw'
            },
            {
                'share_path': 'default_path',
                'access_to': '172.0.0.2',
                'access_level': 'rw'
            }]
        self.assertEqual(expect, result)
        url = 'rest/allNfsShareClient?path=/manila_fakeid/manila_fakeid'
        mock_call.assert_called_once_with(url, None, self.get)

    def test__get_all_cifs_access_rest(self):
        fake_result = {
            'code': 0,
            'message': 'success',
            'data': [
                {
                    'path': 'fake_path',
                    'ugName': 'user_name',
                    'ugType': '0',
                    'accessRight': 'rw'
                },
                {
                    'path': 'default_path',
                    'ugName': 'manilanobody',
                    'ugType': '0',
                    'accessRight': 'rw'
                }]
        }
        mock_call = self.mock_object(self.resthelper,
                                     'call',
                                     mock.Mock(return_value=fake_result))
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        result = self.resthelper._get_all_cifs_access_rest(
            '/manila_fakeid/manila_fakeid')
        expect = [
            {
                'share_path': 'fake_path',
                'access_to': 'user_name',
                'ugType': '0',
                'access_level': 'rw'
            },
            {
                'share_path': 'default_path',
                'access_to': 'manilanobody',
                'ugType': '0',
                'access_level': 'rw'
            }]
        self.assertEqual(expect, result)
        url = 'rest/allCifsShareClient?path=/manila_fakeid/manila_fakeid'
        mock_call.assert_called_once_with(url, None, self.get)

    def test__change_nfs_access_rest(self):
        mock_call = self.mock_object(self.resthelper,
                                     'call')
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        self.resthelper._change_nfs_access_rest(
            '/manila_fakeid/manila_fakeid', '172.0.0.1', 'rw')
        url = 'rest/nfsShareClient'
        data = {
            'path': '/manila_fakeid/manila_fakeid',
            'oldNfsClientName': '172.0.0.1',
            'clientName': '',
            'accessRight': 'rw',
            'allSquash': '',
            'rootSquash': '',
            'secure': '',
            'anonuid': '',
            'anongid': '',
        }
        mock_call.assert_called_once_with(url, data, self.put)

    def test__change_cifs_access_rest(self):
        mock_call = self.mock_object(self.resthelper,
                                     'call')
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        self.resthelper._change_cifs_access_rest(
            '/manila_fakeid/manila_fakeid', '/fake_user', 'rw', '0')
        url = 'rest/cifsShareClient'
        data = {
            'path': '/manila_fakeid/manila_fakeid',
            'right': 'rw',
            'ugName': 'fake_user',
            'ugType': '0',
        }
        mock_call.assert_called_once_with(url, data, self.put)

    def test__delete_nfs_access_rest(self):
        mock_call = self.mock_object(self.resthelper,
                                     'call')
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        self.resthelper._delete_nfs_access_rest(
            '/manila_fakeid/manila_fakeid', '*')
        url = 'rest/nfsShareClient?path=/manila_fakeid/manila_fakeid&client=*'
        mock_call.assert_called_once_with(url, None, self.delete)

    def test__delete_cifs_access_rest(self):
        mock_call = self.mock_object(self.resthelper,
                                     'call')
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        self.resthelper._delete_cifs_access_rest(
            '/manila_fakeid/manila_fakeid', 'fake_user', '0')
        url = 'rest/cifsShareClient?path=/manila_fakeid/manila_fakeid' \
              '&ugName=fake_user&ugType=0'
        mock_call.assert_called_once_with(url, None, self.delete)

    def test__get_nfs_service_status(self):
        fake_result = {
            'code': 0,
            'message': 'success',
            'data': {
                'serviceStatus': constants.NFS_ENABLED,
                'nfs3Status': constants.NFS_SUPPORTED,
                'nfs4Status': constants.NFS_SUPPORTED
            }
        }
        mock_call = self.mock_object(self.resthelper,
                                     'call',
                                     mock.Mock(return_value=fake_result))
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        result = self.resthelper._get_nfs_service_status()
        expect = {
            'serviceStatus': constants.NFS_ENABLED,
            'nfs3Status': constants.NFS_SUPPORTED,
            'nfs4Status': constants.NFS_SUPPORTED
        }
        self.assertEqual(expect, result)
        url = 'rest/nfsService'
        mock_call.assert_called_once_with(url, None, self.get)

    def test__start_nfs_service(self):
        mock_call = self.mock_object(self.resthelper,
                                     'call')
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        self.resthelper._start_nfs_service()
        url = 'rest/nfsService'
        data = {
            "openStatus": "1",
        }
        mock_call.assert_called_once_with(url, data, self.put)

    def test__config_nfs_service(self):
        mock_call = self.mock_object(self.resthelper,
                                     'call')
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        self.resthelper._config_nfs_service()
        url = 'rest/nfsConfig'
        data = {
            'configNfs3': "yes",
            'configNfs4': "yes",
        }
        mock_call.assert_called_once_with(url, data, self.put)

    def test__get_cifs_service_status(self):
        mock_call = self.mock_object(
            self.resthelper,
            'call',
            mock.Mock(return_value=self.result_success_return_1))
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        result = self.resthelper._get_cifs_service_status()
        self.assertEqual('1', result)
        url = 'rest/cifsService'
        mock_call.assert_called_once_with(url, None, self.get)

    def test__start_cifs_service(self):
        mock_call = self.mock_object(self.resthelper,
                                     'call')
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        self.resthelper._start_cifs_service()
        url = 'rest/cifsService'
        data = {
            'openStatus': '1',
        }
        mock_call.assert_called_once_with(url, data, self.put)

    def test__config_cifs_service(self):
        mock_call = self.mock_object(self.resthelper,
                                     'call')
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        self.resthelper._config_cifs_service()
        url = 'rest/cifsConfig'
        data = {
            'workName': 'manila',
            'description': '',
            'access_way': 'user',
            'isCache': 'no',
            'adsName': '',
            'adsIP': '',
            'adsUSER': '',
            'adsPASSWD': '',
            'allowList': [],
            'denyList': [],
        }
        mock_call.assert_called_once_with(url, data, self.put)

    def test__get_all_pool(self):
        mock_call = self.mock_object(
            self.resthelper,
            'call',
            mock.Mock(return_value=self.resutl_success_storage_pools))
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        result = self.resthelper._get_all_pool()
        self.assertEqual(self.resutl_success_storage_pools, result)
        url = 'rest/storagepool'
        mock_call.assert_called_once_with(url, None, self.get)

    def test__query_user(self):
        mock_call = self.mock_object(
            self.resthelper,
            'call',
            mock.Mock(return_value=self.result_success_return_0))
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        result = self.resthelper._query_user('fake_user')
        self.assertEqual('0', result)
        url = 'rest/user/fake_user'
        mock_call.assert_called_once_with(url, None, self.get)

    def test__add_localuser(self):
        mock_call = self.mock_object(self.resthelper,
                                     'call')
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        self.resthelper._add_localuser('fake_user',
                                       'fake_passwd', 'fake_group')
        url = 'rest/localUser'
        data = {
            'userName': 'fake_user',
            'mgGroup': 'fake_group',
            'userPasswd': 'fake_passwd',
            'unusedGroup': []}
        mock_call.assert_called_once_with(url, data, self.post)

    def test__query_group(self):
        mock_call = self.mock_object(
            self.resthelper,
            'call',
            mock.Mock(return_value=self.result_success_return_0))
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        result = self.resthelper._query_group('fake_group')
        self.assertEqual('0', result)
        url = 'rest/group/fake_group'
        mock_call.assert_called_once_with(url, None, self.get)

    def test__add_localgroup(self):
        mock_call = self.mock_object(self.resthelper,
                                     'call')
        self.mock_object(self.resthelper,
                         '_assert_result_code')
        self.resthelper._add_localgroup('fake_group')
        url = 'rest/localGroup'
        data = {'groupName': 'fake_group'}
        mock_call.assert_called_once_with(url, data, self.post)
